Attribute VB_Name = "modBitmaps"
Option Explicit

' Bitmap functions
Declare Function BitBlt Lib "gdi32" (ByVal hDestDC As Long, ByVal x As Long, ByVal y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal hSrcDC As Long, ByVal xSrc As Long, ByVal ySrc As Long, ByVal dwRop As Long) As Long
Declare Function GetDIBits Lib "gdi32" (ByVal aHDC As Long, ByVal hBitmap As Long, ByVal nStartScan As Long, ByVal nNumScans As Long, lpBits As Any, lpBI As BITMAPINFO, ByVal wUsage As Long) As Long
Declare Function StretchDIBits Lib "gdi32" (ByVal hdc As Long, ByVal x As Long, ByVal y As Long, ByVal dx As Long, ByVal dy As Long, ByVal SrcX As Long, ByVal SrcY As Long, ByVal wSrcWidth As Long, ByVal wSrcHeight As Long, lpBits As Any, lpBitsInfo As BITMAPINFO, ByVal wUsage As Long, ByVal dwRop As Long) As Long
Declare Function StretchBlt Lib "gdi32" (ByVal hdc As Long, ByVal x As Long, ByVal y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal hSrcDC As Long, ByVal xSrc As Long, ByVal ySrc As Long, ByVal nSrcWidth As Long, ByVal nSrcHeight As Long, ByVal dwRop As Long) As Long
Declare Function SetStretchBltMode Lib "gdi32" (ByVal hdc As Long, ByVal nStretchMode As Long) As Long

Public Const DIB_RGB_COLORS = 0

Public Const SRCCOPY = &HCC0020
Public Const SRCAND = &H8800C6
Public Const SRCPAINT = &HEE0086

' Pixel functions
Declare Function GetPixel Lib "gdi32" (ByVal hdc As Long, ByVal x As Long, ByVal y As Long) As Long
Declare Function SetPixel Lib "gdi32" (ByVal hdc As Long, ByVal x As Long, ByVal y As Long, ByVal crColor As Long) As Long

' Global Heap functions
Declare Function GlobalAlloc Lib "kernel32" (ByVal wFlags As Long, ByVal dwBytes As Long) As Long
Declare Function GlobalLock Lib "kernel32" (ByVal hMem As Long) As Long
Declare Function GlobalUnlock Lib "kernel32" (ByVal hMem As Long) As Long
Declare Function GlobalFree Lib "kernel32" (ByVal hMem As Long) As Long
'Declare Function MemoryRead Lib "toolhelp" (wSel As Long, dwOffset As Long, lpvBuf As Long, dwcb As Long) As Long

Public Const GMEM_MOVEABLE = &H2
Public Const GMEM_ZEROINIT = &H40

' File functions
Declare Function OpenFile Lib "kernel32" (ByVal lpFileName As String, lpReOpenBuff As OFSTRUCT, ByVal wStyle As Long) As Long
Declare Function hread Lib "kernel32" Alias "_hread" (ByVal hFile As Long, lpBuffer As Any, ByVal lBytes As Long) As Long
Declare Function lclose Lib "kernel32" Alias "_lclose" (ByVal hFile As Long) As Long
Declare Function llseek Lib "kernel32" Alias "_llseek" (ByVal hFile As Long, ByVal lOffset As Long, ByVal iOrigin As Long) As Long

Public Const OF_READ = &H0
Public Const OF_SHARE_DENY_NONE = &H40

' Cursor functions
Declare Function GetCursorPos Lib "user32" (lpPoint As POINTAPI) As Long

' Pen and Brush functions
Declare Function GetObject Lib "gdi32" Alias "GetObjectA" (ByVal hObject As Long, ByVal nCount As Long, lpObject As Any) As Long
Declare Function CreatePen Lib "gdi32" (ByVal nPenStyle As Long, ByVal nWidth As Long, ByVal crColor As Long) As Long
Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, ByVal hObject As Long) As Long
Declare Function Ellipse Lib "gdi32" (ByVal hdc As Long, ByVal X1 As Long, ByVal Y1 As Long, ByVal X2 As Long, ByVal Y2 As Long) As Long
Declare Function CreateSolidBrush Lib "gdi32" (ByVal crColor As Long) As Long
Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long

' Device Mode functions (Display Settings)
Declare Function EnumDisplaySettings Lib "user32" Alias "EnumDisplaySettingsA" (ByVal lpszDeviceName As Long, ByVal iModeNum As Long, lpDevMode As Any) As Boolean
Declare Function ChangeDisplaySettings Lib "user32" Alias "ChangeDisplaySettingsA" (lpDevMode As Any, ByVal dwFlags As Long) As Long

' Variables for determining screen resolution
Declare Function GetSystemMetrics Lib "user32" (ByVal nIndex As Long) As Long

Public Const SM_CXSCREEN = 0
Public Const SM_CYSCREEN = 1

' Variables for Device Mode (to change system color depth and resolution)
Public Const CCDEVICENAME = 32
Public Const CCFORMNAME = 32
Public Const DISP_CHANGE_SUCCESSFUL = 0
Public Const DISP_CHANGE_RESTART = 1
Public Const DISP_CHANGE_FAILED = -1
Public Const DISP_CHANGE_BADMODE = -2
Public Const DISP_CHANGE_NOTUPDATED = -3
Public Const DISP_CHANGE_BADFLAGS = -4
Public Const DISP_CHANGE_BADPARAM = -5
Public Const CDS_UPDATEREGISTRY = &H1
Public Const CDS_TEST = &H2
Public Const DM_BITSPERPEL = &H40000
Public Const DM_PELSWIDTH = &H80000
Public Const DM_PELSHEIGHT = &H100000

Public Type DEVMODE
    dmDeviceName As String * CCDEVICENAME
    dmSpecVersion As Integer
    dmDriverVersion As Integer
    dmSize As Integer
    dmDriverExtra As Integer
    dmFields As Long
    dmOrientation As Integer
    dmPaperSize As Integer
    dmPaperLength As Integer
    dmPaperWidth As Integer
    dmScale As Integer
    dmCopies As Integer
    dmDefaultSource As Integer
    dmPrintQuality As Integer
    dmColor As Integer
    dmDuplex As Integer
    dmYResolution As Integer
    dmTTOption As Integer
    dmCollate As Integer
    dmFormName As String * CCFORMNAME
    dmUnusedPadding As Integer
    dmBitsPerPel As Integer
    dmPelsWidth As Long
    dmPelsHeight As Long
    dmDisplayFlags As Long
    dmDisplayFrequency As Long
End Type

' Types for various Bitmap functions: read a file, determine Palette, etc.
Type POINTAPI
    x As Long
    y As Long
End Type

Type BITMAP
    bmType As Long
    bmWidth As Long
    bmHeight As Long
    bmWidthBytes As Long
    bmPlanes As Integer
    bmBitsPixel As Integer
    bmBits As Long
End Type

Type BITMAPFILEHEADER
    bfType As Integer
    bfSize As Long
    bfReserved1 As Integer
    bfReserved2 As Integer
    bfOffBits As Long
End Type

Type BITMAPINFOHEADER
    biSize As Long
    biWidth As Long
    biHeight As Long
    biPlanes As Integer
    biBitCount As Integer
    biCompression As Long
    biSizeImage As Long
    biXPelsPerMeter As Long
    biYPelsPerMeter As Long
    biClrUsed As Long
    biClrImportant As Long
End Type

Type RGBQUAD
    rgbBlue As Byte
    rgbGreen As Byte
    rgbRed As Byte
    rgbReserved As Byte
End Type

Type BITMAPINFO
    bmiHeader As BITMAPINFOHEADER
    bmiColors(255) As RGBQUAD
End Type

Type OFSTRUCT
    cBytes As Byte
    fFixedDisk As Byte
    nErrCode As Integer
    Reserved1 As Integer
    Reserved2 As Integer
    szPathName(127) As Byte
End Type

Public Function ChangeScreenSettings(lWidth As Integer, lHeight As Integer, lColors As Integer) As Integer
    Dim tDevMode As DEVMODE, lTemp As Long, lIndex As Long
    lIndex = 0
    Do
        lTemp = EnumDisplaySettings(0&, lIndex, tDevMode)
        If lTemp = 0 Then Exit Do
        lIndex = lIndex + 1
        With tDevMode
            If .dmPelsWidth = lWidth And .dmPelsHeight = lHeight And .dmBitsPerPel = lColors Then
'                lTemp = ChangeDisplaySettings(tDevMode, CDS_UPDATEREGISTRY)
                lTemp = ChangeDisplaySettings(tDevMode, 0)
                Exit Do
            End If
        End With
    Loop
    Select Case lTemp
        Case DISP_CHANGE_SUCCESSFUL
'            MsgBox "The display settings changed successfully", vbInformation
        Case DISP_CHANGE_RESTART
            MsgBox "The computer must be restarted in order for the graphics mode to work", vbQuestion
        Case DISP_CHANGE_FAILED
            MsgBox "The display driver failed the specified graphics mode", vbCritical
        Case DISP_CHANGE_BADMODE
            MsgBox "The graphics mode is not supported", vbCritical
        Case DISP_CHANGE_NOTUPDATED
            MsgBox "Unable to write settings to the registry", vbCritical
        Case DISP_CHANGE_BADFLAGS
            MsgBox "You Passed invalid data", vbCritical
    End Select
    ChangeScreenSettings = lTemp
End Function

Public Sub ReadBitmapFile(ByVal FileName As String, _
    bmInfo As BITMAPINFO, hMem As Long, TransparentRGB As Long)
    ' This does indeed read the header and info of a BMP file.
    ' Including (most important here) the Palette information into
    ' the array bmInfo.bmiColors(). The final hread loads the actual
    ' bits to lpMem (a section of memory on the heap).
    Dim rc As Long
    Dim fileStruct As OFSTRUCT
    Dim bmFileHeader As BITMAPFILEHEADER
    Dim hFile As Long, lpMem As Long
    Dim BytesRead As Long, OneByte As Byte
    ' Open the bitmap file
    hFile = OpenFile(FileName, fileStruct, OF_READ Or OF_SHARE_DENY_NONE)
    ' Read the bitmap's file header info
    BytesRead = hread(hFile, bmFileHeader, Len(bmFileHeader))
    ' Read the bitmap's info (including the palette entries)
    BytesRead = hread(hFile, bmInfo, Len(bmInfo))
    ' Read the next byte and the back up a byte
    BytesRead = hread(hFile, OneByte, 1)
    TransparentRGB = RGB(bmInfo.bmiColors(OneByte).rgbRed, bmInfo.bmiColors(OneByte).rgbGreen, bmInfo.bmiColors(OneByte).rgbBlue)
    rc = llseek(hFile, -1, 1)
    ' Free any existing pointed to memory
    rc = GlobalFree(hMem)
    ' Allocate chunk of memory based on size of bitmap
'    hMem = GlobalAlloc(GMEM_MOVEABLE Or GMEM_ZEROINIT, _
'        (CLng(bmInfo.bmiHeader.biWidth / 4) * 4 + 4) * bmInfo.bmiHeader.biHeight)
    hMem = GlobalAlloc(GMEM_MOVEABLE Or GMEM_ZEROINIT, _
        (CLng(bmInfo.bmiHeader.biWidth / 4) * 4 + 4) * bmInfo.bmiHeader.biHeight)
    ' Lock that chunk of memory
    lpMem = GlobalLock(hMem)
    ' Read the bitmap
    BytesRead = hread(hFile, ByVal lpMem, _
        (CLng(bmInfo.bmiHeader.biWidth / 4) * 4 + 4) * bmInfo.bmiHeader.biHeight)
    ' Close the file
    rc = lclose(hFile)
    'Unlock resources
    rc = GlobalUnlock(hMem)
End Sub

Public Sub ChangeColor(ByRef bmInfo As BITMAPINFO, ByRef FromColor As Long, _
    ToRed As Byte, ToGreen As Byte, ToBlue As Byte)
    Dim c As Integer
    For c = 0 To 255
        If RGB(bmInfo.bmiColors(c).rgbRed, bmInfo.bmiColors(c).rgbGreen, bmInfo.bmiColors(c).rgbBlue) = FromColor Then
            bmInfo.bmiColors(c).rgbRed = ToRed
            bmInfo.bmiColors(c).rgbGreen = ToGreen
            bmInfo.bmiColors(c).rgbBlue = ToBlue
        End If
    Next c
End Sub

Public Sub MakeMask(ByRef bmInfo As BITMAPINFO, ByRef Mask As BITMAPINFO, _
    ByVal TransparentColor As Long)
    Dim c As Integer, i As Integer
    Mask = bmInfo
    For c = 0 To 255
        ' Change entry matching TransparentColor to Black in bmInfo
        If RGB(bmInfo.bmiColors(c).rgbRed, bmInfo.bmiColors(c).rgbGreen, bmInfo.bmiColors(c).rgbBlue) = TransparentColor Then
            bmInfo.bmiColors(c).rgbRed = 0
            bmInfo.bmiColors(c).rgbGreen = 0
            bmInfo.bmiColors(c).rgbBlue = 0
        End If
        ' Create mask based on TransparentColor
        If RGB(Mask.bmiColors(c).rgbRed, Mask.bmiColors(c).rgbGreen, Mask.bmiColors(c).rgbBlue) = TransparentColor Then
            Mask.bmiColors(c).rgbRed = 255
            Mask.bmiColors(c).rgbGreen = 255
            Mask.bmiColors(c).rgbBlue = 255
        Else
            Mask.bmiColors(c).rgbRed = 0
            Mask.bmiColors(c).rgbGreen = 0
            Mask.bmiColors(c).rgbBlue = 0
        End If
    Next c
End Sub

Public Sub MakeFadeToWhite(ByRef bmInfo As BITMAPINFO, ByRef bmFadeToWhite() As BITMAPINFO)
    Dim c As Integer, i As Integer
    For c = 0 To UBound(bmFadeToWhite)
        bmFadeToWhite(c) = bmInfo
        For i = 0 To 255
            bmFadeToWhite(c).bmiColors(i).rgbRed = bmInfo.bmiColors(i).rgbRed + (255 - bmInfo.bmiColors(i).rgbRed) * (c / UBound(bmFadeToWhite))
            bmFadeToWhite(c).bmiColors(i).rgbGreen = bmInfo.bmiColors(i).rgbGreen + (255 - bmInfo.bmiColors(i).rgbGreen) * (c / UBound(bmFadeToWhite))
            bmFadeToWhite(c).bmiColors(i).rgbBlue = bmInfo.bmiColors(i).rgbBlue + (255 - bmInfo.bmiColors(i).rgbBlue) * (c / UBound(bmFadeToWhite))
        Next i
    Next c
End Sub

Public Sub MakeGray(ByRef bmInfo As BITMAPINFO, ByRef bmHalfTone As BITMAPINFO, ByVal Percent As Integer)
    Dim c As Integer, i As Byte
    bmHalfTone = bmInfo
    For c = 0 To 255
        i = CByte((CInt(bmInfo.bmiColors(c).rgbRed) + CInt(bmInfo.bmiColors(c).rgbGreen) + _
            CInt(bmInfo.bmiColors(c).rgbBlue)) / 3)
        If i * (Percent / 100) < 255 Then
            i = i * (Percent / 100)
        Else
            i = 255
        End If
        bmHalfTone.bmiColors(c).rgbRed = i
        bmHalfTone.bmiColors(c).rgbGreen = i
        bmHalfTone.bmiColors(c).rgbBlue = i
    Next c
End Sub

Public Sub MakeDark(ByRef bmInfo As BITMAPINFO, ByRef bmDarkTone As BITMAPINFO, ByVal Percent As Integer)
    Dim c As Integer
    bmDarkTone = bmInfo
    For c = 0 To 255
        bmDarkTone.bmiColors(c).rgbRed = bmDarkTone.bmiColors(c).rgbRed * (100 - Percent) / 100
        bmDarkTone.bmiColors(c).rgbGreen = bmDarkTone.bmiColors(c).rgbGreen * (100 - Percent) / 100
        bmDarkTone.bmiColors(c).rgbBlue = bmDarkTone.bmiColors(c).rgbBlue * (100 - Percent) / 100
        If Percent > 30 And bmDarkTone.bmiColors(c).rgbBlue <> 0 Then
            If bmDarkTone.bmiColors(c).rgbBlue + 25 > 255 Then
                 bmDarkTone.bmiColors(c).rgbBlue = 255
            Else
                bmDarkTone.bmiColors(c).rgbBlue = bmDarkTone.bmiColors(c).rgbBlue + 25
            End If
        End If
    Next c
End Sub

Public Sub MakeFadeToBlack(ByRef bmInfo As BITMAPINFO, ByRef bmFadeToBlack() As BITMAPINFO)
    Dim c As Integer, i As Integer
    For c = UBound(bmFadeToBlack) To 0 Step -1
        bmFadeToBlack(c) = bmInfo
        For i = 0 To 255
            bmFadeToBlack(c).bmiColors(i).rgbRed = bmInfo.bmiColors(i).rgbRed * (c / UBound(bmFadeToBlack))
            bmFadeToBlack(c).bmiColors(i).rgbGreen = bmInfo.bmiColors(i).rgbGreen * (c / UBound(bmFadeToBlack))
            bmFadeToBlack(c).bmiColors(i).rgbBlue = bmInfo.bmiColors(i).rgbBlue * (c / UBound(bmFadeToBlack))
        Next i
    Next c
End Sub
