VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Encounter"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
' Encounter is zero or more Events, zero or more Creatures and zero
' or more Items

' Version number for Class
Private myVersion As Integer
' myName is used for reference in Plot only (show in TreeView)
Private myName As String
' myIndex is a unique number for this Encounter in the Map
Private myIndex As Integer
' myFirstEntry is text displayed when you first enter the Encounter
Private myFirstEntry As String
' myFirstEntry is text displayed when you enter second and subsequent times
Private mySecondEntry As String
' Flags:
' Bit 0     &H1     On=Use as Hint?, Off=Don't use as hint
' Bit 1     &H2     On=Have already entered, Off=Have not entered
' Bit 2     &H4     On=Cannot fight here, Off=Can fight here.
' Bit 3     &H8     On=Can Talk here, Off=Cannot talk here
' Bit 4     &H10    On=Can Flee this encounter, Off=Cannot flee
' Bit 5     &H20    On=ReGen on Entry, Off=No auto ReGen
' Bit 6     &H40    On=Dark, Off=Well Lit
' Bit 7     Not used
Private myFlags As Byte
' WallPaper PictureFile (if used)
Private myWallpaper As String
' Extended Encounter Settings (if used)
Private myCombatGrid(bdCombatWidth, bdCombatHeight) As Byte

' Chance to sucessfully flee (if allowed)
Private myChanceToFlee As Byte

' Belongs to Theme (0 is not assigned to any theme)
Private myParentTheme As Integer
' Classificattion (inherits MapStyle from Theme or Map)
' 0 - None, 1 - Safe Area, 2 - Dangerous Area, 3 - Inn/Pub, 4 - Merchant,
' 5 - Meeting Area, 6 - Guard Area, 7 - Temple Area, 8 - Dwelling
Private myClassification As Byte
' ReGen flags are:
' Bit 0     &H1     On=ReGen Creatures, Off=Do not regen them
' Bit 1     &H2     On=ReGen Items, Off=Do not regen them
' Bit 2     &H4     On=ReGen Triggers, Off=Do not regen them
' Bit 3     &H8     On=ReGen Descriptions, Off=Do not regen them
' Bit 4     &H10    On=ReGen Locked (do not change Map layout of Encounter), Off=You can rearange the shape
' Bit 5     &H20    On=Not Active, Off=IsActive
' Bit 6-7           Not used
Private myReGenFlags As Byte

' myTriggers is one or more Triggers in this Encounter
Private myTriggers As New Collection
' myCreatures is one or more Creatures in this Encounter. You are in combat
' with them iff one or more live Creatures exist. You encounter them by
' moving the party into any tile of the Encounter, although each Creature
' has its own location within the Encounter.
Private myCreatures As New Collection
' myItems is one or more Items in this Encounter. They are found by
' searching anywhere in the Encounter iff there are no live and
' unfriendly Creatures.
Private myItems As New Collection

Public Property Get Creatures() As Collection
    Set Creatures = myCreatures
End Property

Public Property Get Items() As Collection
    Set Items = myItems
End Property

Public Property Get Triggers() As Collection
    Set Triggers = myTriggers
End Property

Public Property Get Index() As Integer
    Index = myIndex
End Property

Public Property Let Index(NewIndex As Integer)
    myIndex = NewIndex
End Property

Public Property Get Version() As Integer
    Version = myVersion
End Property

Public Property Let Version(NewValue As Integer)
    myVersion = NewValue
End Property

Public Property Get Name() As String
    Name = Trim(myName)
End Property

Public Property Let Name(NewName As String)
    myName = Trim(NewName)
End Property

Public Property Get FirstEntry() As String
    FirstEntry = Trim(myFirstEntry)
End Property

Public Property Let FirstEntry(NewFirstEntry As String)
    myFirstEntry = Trim(NewFirstEntry)
End Property

Public Property Get SecondEntry() As String
    SecondEntry = Trim(mySecondEntry)
End Property

Public Property Let SecondEntry(NewSecondEntry As String)
    mySecondEntry = Trim(NewSecondEntry)
End Property

Public Property Get Wallpaper() As String
    Wallpaper = Trim(myWallpaper)
End Property

Public Property Let Wallpaper(NewValue As String)
    myWallpaper = Trim(NewValue)
End Property

Public Property Get CombatGrid(x As Integer, y As Integer) As Integer
    CombatGrid = myCombatGrid(Least(Greatest(x, 0), bdCombatWidth), Least(Greatest(y, 0), bdCombatHeight))
End Property

Public Property Let CombatGrid(x As Integer, y As Integer, NewValue As Integer)
    ' 0 = Open, 1 = Blocked, 2 = Party Starting, 3 = Monster Starting
    myCombatGrid(Least(Greatest(x, 0), bdCombatWidth), Least(Greatest(y, 0), bdCombatHeight)) = NewValue
End Property

Public Property Get UseHint() As Integer
    UseHint = (myFlags And &H1) > 0
End Property

Public Property Let UseHint(NewValue As Integer)
    If NewValue Then
        myFlags = myFlags Or &H1
    Else
        myFlags = myFlags And (Not &H1)
    End If
End Property

Public Property Get HaveEntered() As Integer
    HaveEntered = (myFlags And &H2) > 0
End Property

Public Property Let HaveEntered(NewValue As Integer)
    If NewValue Then
        myFlags = myFlags Or &H2
    Else
        myFlags = myFlags And (Not &H2)
    End If
End Property

Public Property Get CanTalk() As Integer
    CanTalk = (myFlags And &H8) > 0
End Property

Public Property Let CanTalk(NewValue As Integer)
    If NewValue Then
        myFlags = myFlags Or &H8
    Else
        myFlags = myFlags And (Not &H8)
    End If
End Property

Public Property Get CanFight() As Integer
    CanFight = (myFlags And &H4) > 0
End Property

Public Property Let CanFight(NewValue As Integer)
    If NewValue Then
        myFlags = myFlags Or &H4
    Else
        myFlags = myFlags And (Not &H4)
    End If
End Property

Public Property Get CanFlee() As Integer
    CanFlee = (myFlags And &H10) > 0
End Property

Public Property Let CanFlee(NewValue As Integer)
    If NewValue Then
        myFlags = myFlags Or &H10
    Else
        myFlags = myFlags And (Not &H10)
    End If
End Property

Public Property Get IsDark() As Integer
    IsDark = (myFlags And &H40) > 0
End Property

Public Property Let IsDark(NewValue As Integer)
    If NewValue Then
        myFlags = myFlags Or &H40
    Else
        myFlags = myFlags And (Not &H40)
    End If
End Property

Public Property Get EncounterPoints() As Integer
    Dim CreatureX As Creature, ItemX As Item
    For Each CreatureX In myCreatures
        EncounterPoints = EncounterPoints + CreatureX.EncounterPoints
    Next CreatureX
    For Each ItemX In myItems
        EncounterPoints = EncounterPoints + ItemX.EncounterPoints
    Next ItemX
End Property

Public Property Get GenerateUponEntry() As Integer
    GenerateUponEntry = (myFlags And &H20) > 0
End Property

Public Property Let GenerateUponEntry(NewValue As Integer)
    If NewValue Then
        myFlags = myFlags Or &H20
    Else
        myFlags = myFlags And (Not &H20)
    End If
End Property

Public Property Get Flags() As Byte
    Flags = myFlags
End Property

Public Property Let Flags(NewValue As Byte)
    myFlags = NewValue
End Property

Public Property Get ChanceToFlee() As Integer
    ChanceToFlee = myChanceToFlee
End Property

Public Property Let ChanceToFlee(NewValue As Integer)
    myChanceToFlee = NewValue
End Property

Public Property Get ParentTheme() As Integer
    ParentTheme = myParentTheme
End Property

Public Property Let ParentTheme(NewValue As Integer)
    myParentTheme = NewValue
End Property

Public Property Get Classification() As Integer
    Classification = myClassification
End Property

Public Property Let Classification(NewValue As Integer)
    myClassification = NewValue
End Property

Public Property Get ReGenFlags() As Integer
    ReGenFlags = myReGenFlags
End Property

Public Property Let ReGenFlags(NewValue As Integer)
    myReGenFlags = NewValue
End Property

Public Property Get ReGenCreatures() As Integer
    ReGenCreatures = (myReGenFlags And &H1) > 0
End Property

Public Property Let ReGenCreatures(NewValue As Integer)
    If NewValue Then
        myReGenFlags = myReGenFlags Or &H1
    Else
        myReGenFlags = myReGenFlags And (Not &H1)
    End If
End Property

Public Property Get ReGenItems() As Integer
    ReGenItems = (myReGenFlags And &H2) > 0
End Property

Public Property Let ReGenItems(NewValue As Integer)
    If NewValue Then
        myReGenFlags = myReGenFlags Or &H2
    Else
        myReGenFlags = myReGenFlags And (Not &H2)
    End If
End Property

Public Property Get ReGenTriggers() As Integer
    ReGenTriggers = (myReGenFlags And &H4) > 0
End Property

Public Property Let ReGenTriggers(NewValue As Integer)
    If NewValue Then
        myReGenFlags = myReGenFlags Or &H4
    Else
        myReGenFlags = myReGenFlags And (Not &H4)
    End If
End Property

Public Property Get ReGenDescription() As Integer
    ReGenDescription = (myReGenFlags And &H8) > 0
End Property

Public Property Let ReGenDescription(NewValue As Integer)
    If NewValue Then
        myReGenFlags = myReGenFlags Or &H8
    Else
        myReGenFlags = myReGenFlags And (Not &H8)
    End If
End Property

Public Property Get ReGenLocked() As Integer
    ReGenLocked = (myReGenFlags And &H10) > 0
End Property

Public Property Let ReGenLocked(NewValue As Integer)
    If NewValue Then
        myReGenFlags = myReGenFlags Or &H10
    Else
        myReGenFlags = myReGenFlags And (Not &H10)
    End If
End Property

Public Property Get IsActive() As Integer
    IsActive = (myReGenFlags And &H20) = 0
End Property

Public Property Let IsActive(NewValue As Integer)
    If NewValue Then
        myReGenFlags = myReGenFlags And (Not &H20)
    Else
        myReGenFlags = myReGenFlags Or &H20
    End If
End Property

Public Function AddCreature() As Creature
    ' Adds an Creature and returns a reference to that Creature
    Dim c As Integer
    Dim CreatureX As Creature
    ' Find new available unused index idenifier
    c = 1
    For Each CreatureX In myCreatures
        If CreatureX.Index >= c Then
            c = CreatureX.Index + 1
        End If
    Next CreatureX
    ' Set the index and add the door. Return the new door's index.
    Set CreatureX = New Creature
    CreatureX.Index = c
    CreatureX.Name = "Creature" & c
    myCreatures.Add Item:=CreatureX, Key:="X" & CreatureX.Index
    Set AddCreature = CreatureX
End Function

Public Sub RemoveCreature(DeleteKey As String)
    myCreatures.Remove DeleteKey
End Sub

Public Function AddItem() As Item
    ' Adds an Item and returns a reference to that Item
    Dim c As Integer
    Dim ItemX As Item
    ' Find new available unused index idenifier
    c = 1
    For Each ItemX In myItems
        If ItemX.Index >= c Then
            c = ItemX.Index + 1
        End If
    Next ItemX
    ' Set the index and add the door. Return the new door's index.
    Set ItemX = New Item
    ItemX.Index = c
    ItemX.Name = "Item" & c
    myItems.Add Item:=ItemX, Key:="I" & ItemX.Index
    Set AddItem = ItemX
End Function

Public Sub RemoveItem(DeleteKey As String)
    myItems.Remove DeleteKey
End Sub

Public Function AddTrigger() As Trigger
    ' Adds an Item and returns a reference to that Item
    Dim c As Integer
    Dim TriggerX As Trigger
    ' Find new available unused index idenifier
    c = 1
    For Each TriggerX In myTriggers
        If TriggerX.Index >= c Then
            c = TriggerX.Index + 1
        End If
    Next TriggerX
    ' Set the index and add the door. Return the new door's index.
    Set TriggerX = New Trigger
    TriggerX.Index = c
    TriggerX.Name = "Trigger" & c
    myTriggers.Add Item:=TriggerX, Key:="T" & TriggerX.Index
    Set AddTrigger = TriggerX
End Function

Public Sub RemoveTrigger(DeleteKey As String)
    myTriggers.Remove DeleteKey
End Sub

Public Sub Copy(FromEncounter As Encounter)
    Dim c As Integer
    Dim myTrigger As Trigger, myItem As Item, myCreature As Creature
    ' Copy Encounter Name
    myName = FromEncounter.Name
    myFirstEntry = FromEncounter.FirstEntry
    mySecondEntry = FromEncounter.SecondEntry
    myWallpaper = FromEncounter.Wallpaper
    myFlags = FromEncounter.Flags
    myChanceToFlee = FromEncounter.ChanceToFlee
    myParentTheme = FromEncounter.ParentTheme
    myClassification = FromEncounter.Classification
    myReGenFlags = FromEncounter.ReGenFlags
    ' Copy Triggers for Encounter
    Set myTriggers = New Collection
    For c = 1 To FromEncounter.Triggers.Count
        Set myTrigger = Me.AddTrigger
        myTrigger.Copy FromEncounter.Triggers(c)
    Next c
    ' Copy Creatures for Encounter
    Set myCreatures = New Collection
    For c = 1 To FromEncounter.Creatures.Count
        Set myCreature = Me.AddCreature
        myCreature.Copy FromEncounter.Creatures(c)
    Next c
    ' Copy Items for Encounter
    Set myItems = New Collection
    For c = 1 To FromEncounter.Items.Count
        Set myItem = Me.AddItem
        myItem.Copy FromEncounter.Items(c)
    Next c
End Sub

Public Sub SaveToFile(ToFile As Integer)
    Dim c As Integer, tmp As String, x As Integer, y As Integer
    ' Save Encounter Name
    Put ToFile, , myVersion
    Put ToFile, , Len(myName)
    Put ToFile, , myName
    Put ToFile, , myIndex
    Put ToFile, , Len(myFirstEntry)
    Put ToFile, , myFirstEntry
    Put ToFile, , Len(mySecondEntry)
    Put ToFile, , mySecondEntry
    If Len(myWallpaper) > 0 Then
        tmp = myWallpaper & "|"
        For x = 0 To bdCombatWidth: For y = 0 To bdCombatHeight
            tmp = tmp & Format(myCombatGrid(x, y))
        Next y: Next x
        Put ToFile, , Len(tmp)
        Put ToFile, , tmp
    Else
        Put ToFile, , Len(myWallpaper)
        Put ToFile, , myWallpaper
    End If
    Put ToFile, , myFlags
    Put ToFile, , myChanceToFlee
    Put ToFile, , myParentTheme
    Put ToFile, , myClassification
    Put ToFile, , myReGenFlags
    ' Save Triggers for Encounter
    Put ToFile, , myTriggers.Count
    For c = 1 To myTriggers.Count
        myTriggers(c).SaveToFile ToFile
    Next c
    ' Save Creatures for Encounter
    Put ToFile, , myCreatures.Count
    For c = 1 To myCreatures.Count
        myCreatures(c).SaveToFile ToFile
    Next c
    ' Save Items for Encounter
    Put ToFile, , myItems.Count
    For c = 1 To myItems.Count
        myItems(c).SaveToFile ToFile
    Next c
End Sub

Public Sub ReadFromFile(FromFile As Integer)
    Dim c As Integer, x As Integer, y As Integer, Cnt As Long
    Dim myTrigger As Trigger, myCreature As Creature
    Dim tmp As String
    Dim myItem As Item
    ' Read Encounter Name and Index
    Get FromFile, , myVersion
    Get FromFile, , Cnt
    myName = ""
    For c = 1 To Cnt
        myName = myName & " "
    Next c
    Get FromFile, , myName
    Get FromFile, , myIndex
    ' Read Entry Text
    Get FromFile, , Cnt
    myFirstEntry = ""
    For c = 1 To Cnt
        myFirstEntry = myFirstEntry & " "
    Next c
    Get FromFile, , myFirstEntry
    ' Read Second Entry Text (if any)
    Get FromFile, , Cnt
    mySecondEntry = ""
    For c = 1 To Cnt
        mySecondEntry = mySecondEntry & " "
    Next c
    Get FromFile, , mySecondEntry
    ' Read Wallpaper PictureFile (if any)
    Get FromFile, , Cnt
    myWallpaper = ""
    For c = 1 To Cnt
        myWallpaper = myWallpaper & " "
    Next c
    Get FromFile, , myWallpaper
    ' Check for extended Encounter combat settings
    tmp = modBD.BreakText(myWallpaper, 2)
    If Len(tmp) > 0 Then
        c = 1
        For x = 0 To bdCombatWidth: For y = 0 To bdCombatHeight
            If IsNumeric(Mid(tmp, c, 1)) = True Then
                myCombatGrid(x, y) = CByte(Mid(tmp, c, 1))
            Else
                myCombatGrid(x, y) = 0
            End If
            c = c + 1
        Next y: Next x
        myWallpaper = modBD.BreakText(myWallpaper, 1)
    End If
    Get FromFile, , myFlags
    Get FromFile, , myChanceToFlee
    Get FromFile, , myParentTheme
    Get FromFile, , myClassification
    Get FromFile, , myReGenFlags
    ' Read Triggers for Encounter
    Get FromFile, , Cnt
    For c = 1 To Cnt
        Set myTrigger = New Trigger
        myTrigger.ReadFromFile FromFile
        myTriggers.Add Item:=myTrigger, Key:="T" & myTrigger.Index
    Next c
    ' Read Creatures for Encounter
    Get FromFile, , Cnt
    For c = 1 To Cnt
        Set myCreature = New Creature
        myCreature.ReadFromFile FromFile
        myCreatures.Add Item:=myCreature, Key:="X" & myCreature.Index
    Next c
    ' Read Items for Encounter
    Get FromFile, , Cnt
    For c = 1 To Cnt
        Set myItem = New Item
        myItem.ReadFromFile FromFile
        myItems.Add Item:=myItem, Key:="I" & myItem.Index
    Next c
End Sub

Private Sub Class_Initialize()
    Me.CanFight = True
    Me.CanFlee = True
    Me.ChanceToFlee = 100
    Me.Name = "BlankEncounter"
End Sub
