VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Item"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' Version number for Item
Private myVersion As Integer
' Unique index for this Item
Private myIndex As Integer
' Name of this Item
Private myName As String
' Comments for this Item (can be used as Description)
Private myComments As String
' See SetUpItemFamily in modBD.
Private myFamily As Byte
'   &H1 Bit 1   - On=Use Comments as Description, Off=Don't
'   &H2 Bit 2   - On=Soft Bulk (as Bulky as Bulkiest Item within), Off=Hard Bulk
'   &H4 Bit 3   - On=Can Combine when match name, Off=Cannot combine
'   Bit 4-8     - Key Type (32 varieties, used only with Key Family or Container Family items).
Private myFlags As Byte
' myCount is the number of Items this Item represents. Like Items
' (with the EXACT same name) can be combined. myCount is normally
' just 1.
Private myCount As Integer
' Value in BP of this Item
Private myValue As Integer
' myWeight is the weight of this Item NOT including Contents. To
' get the "total" weight, you must add up the Items weight plus
' myWeight. Characters can carry Strength * 25 in Bilos (100 BP is 1 b, Strength 10 carry 2500 BP).
Private myWeight As Integer
' myBulk is the bulk of this Item NOT including Contents. If flagged
' as "soft capacity," the total bulk this Item represents is the
' bulk of all items inside this one.
' Characters can carry Agility * 15 in Cubic Bilos (cb) (10 BP is 1 cb)
Private myBulk As Integer
' Capacity (Total Capacity - 0 is no capacity, cannot be opened; 255 is infinite capacity)
Private myCapacity As Integer
' Used only in UberWizard and while Dragging
Private mySelected As Integer
'   Bit 1-2     - 0 - Short, 1 - Medium, 2- Long Range, 3 - Undefined
'   Bit 3-4     - 0 - Melee, 1 - Ranged, 2 - Ammo, 3 - Thrown
'   Bit 5-7     - Ammo Type - Arrow (Short), Arrow (Long), Sling (Light), Sling (Heavy),
'                             Gun (Small), Gun (Large), Energy (Low), Energy (High)
'   Bit 8       - On = IsSelected (UberWiz and Inventory)
Private myShootType As Byte

' Defines coordinate in Inventory screen. (0 = Not Placed, 1-255 = Inventory/Container)
Private myInvSpot As Byte

Private myNotUsed As Integer

' Weapon ActionPoints is how much ActionPoints it takes to use
Private myActionPoints As Integer

' Run-time only
Private myPic As Byte
' Name of the image file for this item (in /item directory)
Private myPictureFile As String
' X and Y on Map for Encounter. Not used on Creatures.
Private myMapX As Integer
Private myMapY As Integer
' A number from 0 to 4 indicating the spot on the Tile.
Private myTileSpot As Byte

' Bits 1-4     - Type of Resistance (Armor) or Damage (Weapons)
'       0 - None, 1 - Sharp, 2 - Blunt, 3 - Cold, 4 - Fire, 5 - Evil
'       6 - Holy, 7 - Magic, 8 - Mind
' Bit 5-8       - Amount of Resistance (0-15)
Private myResistanceType As Byte

'   Bit 1-4    - WearType (Armor Only)
'       0 - Body, 1 - Helm, 2 - Glove, 3 - Bracelet, 4 - Backpack, 5 - Shield,
'       6 - Boots, 7 - Necklace, 8 - Belt, 9 - Ring, 10 - One Hand, 11 - Two Handed,
'       12-15 - Undefined
'   Bit 5-14    - Not Used
'   Bit 15      - On = IsReady
Private myWearType As Integer
' Amount of dice of damage will do if you in Combat
Private myDamage As Integer
' Amount added to d20 when attempting to hit
Private myAttackBonus As Integer
' Amount of Resistance provided by Item when wearing
Private myResistance As Integer
' Amount of Defense this item provides if equipped in hand.
Private myDefense As Integer
' Amount of additional damage from the Item
Private myDamageBonus As Integer

' Items contained in this Item
Private myItems As New Collection
' Triggers attached to this Item
Private myTriggers As New Collection

Public Function AddTrigger() As Trigger
    ' Adds an Item and returns a reference to that Item
    Dim c As Integer
    Dim TriggerX As Trigger
    ' Find new available unused index idenifier
    c = 1
    For Each TriggerX In myTriggers
        If TriggerX.Index >= c Then
            c = TriggerX.Index + 1
        End If
    Next TriggerX
    ' Set the index and add the door. Return the new door's index.
    Set TriggerX = New Trigger
    TriggerX.Index = c
    TriggerX.Name = "Trigger" & c
    myTriggers.Add Item:=TriggerX, Key:="T" & TriggerX.Index
    Set AddTrigger = TriggerX
End Function

Public Sub RemoveTrigger(DeleteKey As String)
    myTriggers.Remove DeleteKey
End Sub

Public Function AddItem() As Item
    ' Adds an Item and returns a reference to that Item
    Dim c As Integer
    Dim ItemX As Item
    ' Find new available unused index idenifier
    c = 1
    For Each ItemX In myItems
        If ItemX.Index >= c Then
            c = ItemX.Index + 1
        End If
    Next ItemX
    ' Set the index and add the door. Return the new door's index.
    Set ItemX = New Item
    ItemX.Index = c
    ItemX.Name = "Item" & c
    myItems.Add Item:=ItemX, Key:="I" & ItemX.Index
    Set AddItem = ItemX
End Function

Public Function AddItemBefore(BeforeKey As String) As Item
    ' Adds an Item and returns a reference to that Item
    Dim c As Integer
    Dim ItemX As Item
    ' Find new available unused index idenifier
    c = 1
    For Each ItemX In myItems
        If ItemX.Index >= c Then
            c = ItemX.Index + 1
        End If
    Next ItemX
    ' Set the index and add the door. Return the new door's index.
    Set ItemX = New Item
    ItemX.Index = c
    ItemX.Name = "Item" & c
    myItems.Add Item:=ItemX, Key:="I" & ItemX.Index, Before:=BeforeKey
    Set AddItem = ItemX
End Function

Public Sub RemoveItem(DeleteKey As String)
    myItems.Remove DeleteKey
End Sub

Public Property Get Index() As Integer
    Index = myIndex
End Property

Public Property Let Index(NewIndex As Integer)
    myIndex = NewIndex
End Property

Public Property Get Name() As String
    If myCount > 1 Then
        Name = Format(myCount) & " " & Trim(myName) & "s"
    Else
        Name = Trim(myName)
    End If
End Property

Public Property Let Name(NewValue As String)
    myName = Trim(NewValue)
End Property

Public Property Get NameText() As String
    NameText = Trim(myName)
End Property

Public Property Get PictureFile() As String
    PictureFile = Trim(myPictureFile)
End Property

Public Property Let PictureFile(NewValue As String)
    myPictureFile = Trim(NewValue)
End Property

Public Property Get Comments() As String
    Comments = Trim(myComments)
End Property

Public Property Let Comments(NewComments As String)
    myComments = Trim(NewComments)
End Property

Public Property Get Hint() As String
    If Me.WearType < 10 Then   ' Potential Armor
        Hint = " * Protect " & Me.Resistance & "%"
    ElseIf Me.Damage > 0 Then    ' Potential Weapon
        Hint = " * Damage " & (Me.Damage - 1) Mod 5 + 1 & "d" & Int(((Me.Damage - 1) Mod 25) / 5) * 2 + 4
        If Me.Damage - 1 > 24 Then
            Hint = Hint & "+" & Int((Me.Damage - 1) / 25)
        End If
    End If
    Hint = Me.Name & " * Bulk " & Me.Bulk & "  Wgt " & Me.Weight & Hint
End Property

Public Property Get Family() As Integer
    Family = myFamily
End Property

Public Property Let Family(NewFamily As Integer)
    myFamily = NewFamily
End Property

Public Property Get UseDescription() As Integer
    UseDescription = (myFlags And &H1) > 0
End Property

Public Property Let UseDescription(NewValue As Integer)
    If NewValue Then
        myFlags = myFlags Or &H1
    Else
        myFlags = myFlags And (Not &H1)
    End If
End Property

Public Property Get CanCombine() As Integer
    CanCombine = (myFlags And &H4) > 0
End Property

Public Property Let CanCombine(NewValue As Integer)
    If NewValue Then
        myFlags = myFlags Or &H4
    Else
        myFlags = myFlags And (Not &H4)
    End If
End Property

Public Property Get IsSelected() As Integer
    IsSelected = (myShootType And &H80) > 0
End Property

Public Property Let IsSelected(NewValue As Integer)
    If NewValue Then
        myShootType = myShootType Or &H80
    Else
        myShootType = myShootType And (Not &H80)
    End If
End Property

Public Property Get SoftCapacity() As Integer
    SoftCapacity = (myFlags And &H2) > 0
End Property

Public Property Let SoftCapacity(NewValue As Integer)
    If NewValue Then
        myFlags = myFlags Or &H2
    Else
        myFlags = myFlags And (Not &H2)
    End If
End Property

Public Property Get Count() As Integer
    Count = myCount
End Property

Public Property Let Count(NewCount As Integer)
    myCount = NewCount
    If myCount < 1 Then
        myCount = 1
    End If
End Property

Public Property Get Value() As Integer
    Value = myValue
End Property

Public Property Let Value(NewValue As Integer)
    myValue = NewValue
End Property

Public Property Get EncounterPoints() As Integer
    Dim TriggerX As Trigger
    EncounterPoints = myValue
    For Each TriggerX In myTriggers
        EncounterPoints = EncounterPoints + 5 * TriggerX.Statements.Count
    Next TriggerX
End Property

Public Property Get WeightEmpty() As Integer
    WeightEmpty = myWeight
End Property

Public Property Get Weight() As Integer
    Dim ItemX As Item, c As Integer
    c = myWeight
    ' If Item weighs something, then add what's inside as well
    If myWeight > 0 Then
        For Each ItemX In myItems
            c = c + ItemX.Weight
        Next ItemX
    End If
    Weight = c
End Property

Public Property Let Weight(NewWeight As Integer)
    myWeight = NewWeight
End Property

Public Property Get Full() As Integer
    Dim ItemX As Item
    Full = 0
    For Each ItemX In myItems
        Full = Full + ItemX.Bulk
    Next ItemX
End Property

Public Property Get Money() As Long
    Dim ItemX As Item
    Money = 0
    If Me.Family = 2 Then   ' Money Family
        Money = Me.Value
    End If
    For Each ItemX In Me.Items
        Money = Money + ItemX.Money
    Next ItemX
End Property

Public Function RemoveMoney(AmountToRemove As Integer) As Integer
    Dim Take As Double, ItemX As Item
    ' Remove from Contents
    For Each ItemX In Me.Items
        If ItemX.RemoveMoney(AmountToRemove) = True Then
            Me.RemoveItem "I" & ItemX.Index
        End If
    Next ItemX
    ' Remove Me.Value if asking for omre
    RemoveMoney = False
    If Me.IsMoney = True And AmountToRemove > 0 Then
        If myValue > 0 Then
            Take = Int(AmountToRemove / (myValue / myCount))
            If myBulk > 0 Then
                myBulk = myBulk - Take * Int(myBulk / myCount)
            End If
            If myWeight > 0 Then
                myWeight = myWeight - Take * Int(myWeight / myCount)
            End If
            If myCount > 0 Then
                myCount = myCount - Take
            End If
            myValue = myValue - AmountToRemove
            AmountToRemove = AmountToRemove - myValue
        End If
        If myValue < 1 Then
            RemoveMoney = True
        End If
    End If
End Function

Public Property Get Bulk() As Integer
    Dim ItemX As Item
    ' By default, bulk is the bulk of the item.
    Bulk = myBulk
    ' If soft capacity, then Bulk is the biggest thing inside the item.
    If Me.SoftCapacity = True Then
        For Each ItemX In myItems
            If ItemX.Bulk > Bulk Then
                Bulk = ItemX.Bulk
            End If
        Next ItemX
    End If
    If Bulk < 0 Then
        Bulk = 0
    End If
End Property

Public Property Let Bulk(NewValue As Integer)
    myBulk = NewValue
End Property

Public Property Get BulkEmpty() As Integer
    BulkEmpty = myBulk
End Property

Public Property Get Capacity() As Integer
    Capacity = myCapacity
End Property

Public Property Let Capacity(NewValue As Integer)
    myCapacity = NewValue
End Property

Public Property Get Selected() As Integer
    Selected = mySelected
End Property

Public Property Let Selected(NewValue As Integer)
    mySelected = NewValue
End Property

Public Property Get MapX() As Integer
    MapX = myMapX
End Property

Public Property Let MapX(NewValue As Integer)
    myMapX = NewValue
End Property

Public Property Get MapY() As Integer
    MapY = myMapY
End Property

Public Property Let MapY(NewValue As Integer)
    myMapY = NewValue
End Property

Public Property Get InvSpot() As Integer
    InvSpot = myInvSpot
End Property

Public Property Let InvSpot(NewValue As Integer)
    myInvSpot = NewValue
End Property

Public Property Get TileSpot() As Integer
    TileSpot = myTileSpot
End Property

Public Property Let TileSpot(NewValue As Integer)
    myTileSpot = NewValue
End Property

Public Property Get TileSpotX() As Integer
    Select Case myTileSpot
        Case 0: TileSpotX = 32
        Case 1: TileSpotX = 8
        Case 2: TileSpotX = 32
        Case 3: TileSpotX = 56
        Case 4: TileSpotX = 32
    End Select
End Property

Public Property Get TileSpotY() As Integer
    Select Case myTileSpot
        Case 0: TileSpotY = 40
        Case 1: TileSpotY = 52
        Case 2: TileSpotY = 52
        Case 3: TileSpotY = 52
        Case 4: TileSpotY = 64
    End Select
End Property

Public Property Get Pic() As Integer
    Pic = myPic
End Property

Public Property Let Pic(NewValue As Integer)
    If NewValue > 255 Then NewValue = 255
    If NewValue < 0 Then NewValue = 0
    myPic = NewValue
End Property

Public Property Get Resistance() As Integer
    Resistance = myResistance
End Property

Public Property Let Resistance(NewValue As Integer)
    myResistance = NewValue
End Property

Public Property Get Flags() As Byte
    Flags = myFlags
End Property

Public Property Let Flags(NewValue As Byte)
    myFlags = NewValue
End Property

Public Property Get KeyBit(Index As Integer) As Integer
    KeyBit = (myFlags And (2 ^ Index)) > 0
End Property

Public Property Let KeyBit(Index As Integer, NewValue As Integer)
    If NewValue Then
        myFlags = myFlags Or (2 ^ Index)
    Else
        myFlags = myFlags And (Not (2 ^ Index))
    End If
End Property

Public Property Get KeyBits() As Byte
    KeyBits = (myFlags And &HF8)
End Property

Public Property Let KeyBits(NewValue As Byte)
    ' Wipe out the old key
    KeyBits = (myFlags And &H7)
    ' Set the new key
    KeyBits = (myFlags Or NewValue)
End Property

Public Property Get ShootType() As Integer
    ShootType = Abs((myShootType And &H10) > 0) + Abs((myShootType And &H20) > 0) * 2 + Abs((myShootType And &H40) > 0) * 4
End Property

Public Property Let ShootType(NewValue As Integer)
    ' Clear previous value
    myShootType = myShootType And (Not &H70)
    Select Case NewValue
        Case 1, 3, 5, 7
            ' 001
            myShootType = myShootType Or &H10
    End Select
    Select Case NewValue
        Case 2, 3, 6, 7
            ' 010
            myShootType = myShootType Or &H20
    End Select
    Select Case NewValue
        Case 4, 5, 6, 7
            ' 100
            myShootType = myShootType Or &H40
    End Select
End Property

Public Property Get RangeType() As Integer
    RangeType = myShootType And &H3
    If RangeType > 2 Then
        RangeType = 2
    End If
End Property

Public Property Let RangeType(NewValue As Integer)
    ' Clear out previous value
    myShootType = myShootType And (Not &H3)
    Select Case NewValue
        Case 1
            myShootType = myShootType Or &H1
        Case 2
            myShootType = myShootType Or &H2
    End Select
End Property

Public Property Get ShootTypes() As Integer
    ShootTypes = myShootType
End Property

Public Property Let ShootTypes(NewValue As Integer)
    myShootType = NewValue
End Property

Public Property Get ResistanceBonus() As Integer
    ResistanceBonus = Abs((myResistanceType And &H10) > 0) + Abs((myResistanceType And &H20) > 0) * 2 + Abs((myResistanceType And &H40) > 0) * 4 + Abs((myResistanceType And &H80) > 0) * 8
End Property

Public Property Let ResistanceBonus(NewValue As Integer)
    ' Clear current value
    myResistanceType = myResistanceType And (Not &HF0)
    Select Case NewValue
        Case 1, 3, 5, 7, 9, 11, 13, 15
            ' 0001
        myResistanceType = myResistanceType Or &H10
    End Select
    Select Case NewValue
        Case 2, 3, 6, 7, 10, 11, 14, 15
            ' 0010
        myResistanceType = myResistanceType Or &H20
    End Select
    Select Case NewValue
        Case 4, 5, 6, 7, 12, 13, 14, 15
            ' 0100
        myResistanceType = myResistanceType Or &H40
    End Select
    Select Case NewValue
        Case 8, 9, 10, 11, 12, 13, 14, 15
            ' 1000
        myResistanceType = myResistanceType Or &H80
    End Select
End Property

Public Property Get MaxRange() As Integer
    ' This is the range in grid squares of this item
    Select Case (myShootType And &H3)
        Case 0  ' Short
            MaxRange = 1
        Case 1  ' Medium
            MaxRange = 2
        Case 2  ' Long
            MaxRange = 50
        Case Else
            MaxRange = 1
    End Select
End Property

Public Property Get MinRange() As Integer
    ' This is the range in grid squares of this item
    Select Case (myShootType And &H3)
        Case 0  ' Short
            MinRange = 1
        Case 1  ' Medium
            MinRange = 1
        Case 2  ' Long
            MinRange = 3
        Case Else
            MinRange = 1
    End Select
End Property

Public Property Get IsReady() As Integer
    IsReady = (myWearType And &H4000) > 0
End Property

Public Property Let IsReady(NewValue As Integer)
    If NewValue Then
        myWearType = myWearType Or &H4000
    Else
        myWearType = myWearType And (Not &H4000)
    End If
End Property

Public Property Get IsMoney() As Integer
    IsMoney = (myFamily = 2)
End Property

Public Property Let IsMoney(Value As Integer)
    If Value = True Then
        myFamily = 2
    ElseIf myFamily = 2 Then
        myFamily = 0
    End If
End Property

Public Property Get InHand() As Integer
    InHand = ((Me.WearType = 10 Or Me.WearType = 11) And Me.IsReady = True)
End Property

Public Property Get IsAmmo() As Integer
    IsAmmo = (Me.Family = 6)
End Property

Public Property Get IsShooter() As Integer
    IsShooter = (Me.Family = 7)
End Property

Public Property Get WearTypes() As Integer
    WearTypes = myWearType
End Property

Public Property Get WearType() As Integer
    If myFamily > 3 And myFamily < 9 Then   ' Armor or Weapon
        WearType = myWearType And &HF
        If WearType > 11 Then
            WearType = 10
        End If
    Else
        WearType = 10
    End If
End Property

Public Property Let WearType(NewValue As Integer)
    ' Clear current WearType
    myWearType = myWearType And (Not &HF)
    ' Set new WearType
    myWearType = (myWearType Or (NewValue And &HF))
End Property

Public Property Get Damage() As Integer
    Damage = myDamage
End Property

Public Property Let Damage(NewDamage As Integer)
    myDamage = NewDamage
End Property

Public Property Get AttackBonus() As Integer
    AttackBonus = myAttackBonus
End Property

Public Property Let AttackBonus(NewValue As Integer)
    myAttackBonus = NewValue
End Property

Public Property Get DamageBonus() As Integer
    DamageBonus = myDamageBonus
End Property

Public Property Let DamageBonus(NewValue As Integer)
    myDamageBonus = NewValue
End Property

Public Property Get DamageType() As Integer
    If myFamily > 4 And myFamily < 9 Then
        DamageType = myResistanceType And &HF
    Else
        DamageType = 0
    End If
End Property

Public Property Let DamageType(NewValue As Integer)
    ' Clear current value
    myResistanceType = myResistanceType And (Not &HF)
    ' Set new value
    If myFamily > 4 And myFamily < 9 Then
        myResistanceType = myResistanceType Or (NewValue And &HF)
    End If
End Property

Public Property Get ResistanceTypeReal() As Byte
    ResistanceTypeReal = myResistanceType
End Property

Public Property Let ResistanceTypeReal(NewValue As Byte)
    myResistanceType = NewValue
End Property

Public Property Get ResistanceType() As Integer
    If myFamily = 4 Then
        ResistanceType = myResistanceType And &HF
    Else
        ResistanceType = 0
    End If
End Property

Public Property Let ResistanceType(NewValue As Integer)
    ' Clear current value
    myResistanceType = myResistanceType And (Not &HF)
    ' Set new value
    If myFamily = 4 Then
        myResistanceType = myResistanceType Or (NewValue And &HF)
    End If
End Property

Public Property Get Items() As Collection
    Set Items = myItems
End Property

Public Property Get Triggers() As Collection
    Set Triggers = myTriggers
End Property

Public Property Get Defense() As Integer
    Defense = myDefense
End Property

Public Property Let Defense(NewValue As Integer)
    myDefense = NewValue
End Property

Public Property Get ActionPoints() As Integer
    If myActionPoints < 0 Then
        ActionPoints = 0
    Else
        ActionPoints = myActionPoints
    End If
End Property

Public Property Let ActionPoints(NewValue As Integer)
    myActionPoints = NewValue
End Property

Public Sub Copy(FromItem As Item)
    Dim c As Integer
    Dim myTrigger As Trigger, myItem As Item
    ' Copy Item Name,and Comments
    myName = FromItem.NameText
    myComments = FromItem.Comments
    ' Copy Attributes
    myFamily = FromItem.Family
    myFlags = FromItem.Flags
    myCount = FromItem.Count
    myValue = FromItem.Value
    myWeight = FromItem.WeightEmpty
    myBulk = FromItem.BulkEmpty
    myCapacity = FromItem.Capacity
    mySelected = FromItem.Selected
    myResistanceType = FromItem.ResistanceTypeReal
    myResistance = FromItem.Resistance
    myDefense = FromItem.Defense
    myShootType = FromItem.ShootTypes
    myActionPoints = FromItem.ActionPoints
    myWearType = FromItem.WearTypes
    myDamage = FromItem.Damage
    myAttackBonus = FromItem.AttackBonus
    myDamageBonus = FromItem.DamageBonus
    myDefense = FromItem.Defense
    myPictureFile = FromItem.PictureFile
    myPic = FromItem.Pic
    ' Copy Triggers for Item
    Set myTriggers = New Collection
    For c = 1 To FromItem.Triggers.Count
        Set myTrigger = Me.AddTrigger
        myTrigger.Copy FromItem.Triggers(c)
    Next c
    ' Copy Items for Item
    Set myItems = New Collection
    For c = 1 To FromItem.Items.Count
        Set myItem = Me.AddItem
        myItem.Copy FromItem.Items(c)
    Next c
End Sub

Public Sub ReadFromFileHeader(FromFile As Integer)
    Dim c As Integer, Cnt As Long
    ' Read Item Name, Index and Comments
    Get FromFile, , myVersion
    Get FromFile, , Cnt
    myName = ""
    For c = 1 To Cnt
        myName = myName & " "
    Next c
    Get FromFile, , myName
    Get FromFile, , myIndex
    ' Read Comments
    Get FromFile, , Cnt
    myComments = ""
    For c = 1 To Cnt
        myComments = myComments & " "
    Next c
    Get FromFile, , myComments
End Sub

Public Sub ReadFromFile(FromFile As Integer)
    Dim c As Integer, Cnt As Long
    Dim myTrigger As Trigger, myItem As Item
    ReadFromFileHeader FromFile
    ' Read Attributes
    Get FromFile, , myFamily
    Get FromFile, , myFlags
    Get FromFile, , myCount
    Get FromFile, , myValue
    Get FromFile, , myWeight
    Get FromFile, , myBulk
    Get FromFile, , myCapacity
    Get FromFile, , myDamageBonus
    Get FromFile, , myNotUsed
    Get FromFile, , myShootType
    Get FromFile, , myResistanceType
    Get FromFile, , myInvSpot
    Get FromFile, , myWearType
    Get FromFile, , myDamage
    Get FromFile, , myAttackBonus
    Get FromFile, , myDefense
    Get FromFile, , myMapX
    Get FromFile, , myMapY
    Get FromFile, , myActionPoints
    Get FromFile, , myTileSpot
    Get FromFile, , myResistance
    Get FromFile, , myDefense
    Get FromFile, , Cnt
    myPictureFile = ""
    For c = 1 To Cnt
        myPictureFile = myPictureFile & " "
    Next c
    Get FromFile, , myPictureFile
    ' Read Triggers for Item
    Get FromFile, , Cnt
    For c = 1 To Cnt
        Set myTrigger = New Trigger
        myTrigger.ReadFromFile FromFile
        myTriggers.Add Item:=myTrigger, Key:="T" & myTrigger.Index
    Next c
    ' Read Items for Item
    Get FromFile, , Cnt
    For c = 1 To Cnt
        Set myItem = New Item
        myItem.ReadFromFile FromFile
        myItems.Add Item:=myItem, Key:="I" & myItem.Index
    Next c
End Sub

Public Sub SaveToFile(ToFile As Integer)
    Dim c As Integer
    ' Save Item Name, Index and Comments
    Put ToFile, , myVersion
    Put ToFile, , Len(myName)
    Put ToFile, , myName
    Put ToFile, , myIndex
    Put ToFile, , Len(myComments)
    Put ToFile, , myComments
    ' Save Attributes
    Put ToFile, , myFamily
    Put ToFile, , myFlags
    Put ToFile, , myCount
    Put ToFile, , myValue
    Put ToFile, , myWeight
    Put ToFile, , myBulk
    Put ToFile, , myCapacity
    Put ToFile, , myDamageBonus
    Put ToFile, , myNotUsed
    Put ToFile, , myShootType
    Put ToFile, , myResistanceType
    Put ToFile, , myInvSpot
    Put ToFile, , myWearType
    Put ToFile, , myDamage
    Put ToFile, , myAttackBonus
    Put ToFile, , myDefense
    Put ToFile, , myMapX
    Put ToFile, , myMapY
    Put ToFile, , myActionPoints
    Put ToFile, , myTileSpot
    Put ToFile, , myResistance
    Put ToFile, , myDefense
    Put ToFile, , Len(myPictureFile)
    Put ToFile, , myPictureFile
    ' Save Triggers for Item
    Put ToFile, , myTriggers.Count
    For c = 1 To myTriggers.Count
        myTriggers(c).SaveToFile ToFile
    Next c
    ' Save Items for Item
    Put ToFile, , myItems.Count
    For c = 1 To myItems.Count
        myItems(c).SaveToFile ToFile
    Next c
End Sub

Private Sub Class_Initialize()
    myName = "Item"
    myCount = 1
    myActionPoints = 10
End Sub
