VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Map"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
' Map is an Object which holds one set of MapLayers, zero or
' more Doors, zero or more Encounters, etc.

' Version number of Class
Private myVersion As Integer
' myName is used for reference in Plot only (show in TreeView)
Private myName As String
' myIndex is a unique number for this Map in the Plot
Private myIndex As Integer
' myComments are text comments used at Design Time only
Private myComments As String

' PictureFile is the file name of the graphic. Looks in /tome first,
' then in /data.
Private myPictureFile As String
' Pointer to PictureBox (used only at RUN time, not saved)
Private myPic As Integer

' ReGen: Style to use in making map
' 0 - Tetris Spin
Private myDefaultStyle As Byte
' RegEn: Total number of Skill points to use in creating this map
Private myExperiencePoints As Integer
' Created for Party Size
' Bit1-2 - 0 = Solo, 1 = 2-3, 2 = 4-6, 3 = 7-9
' With Average Party Level
' Bit3-4 - 0 = 1-3, 1 = 4-6, 2 = 7-10, 3 = 10+
' Bit5  - Off=Normal TileSet, On=No Tile Set
' Bit6  - Off=Use Runes, On=Don't Use Runes
' Bit7-8   Not Used
Private myStyle As Byte
' Flags:
' Bit1  - On=Generate upon Entry Off=Don't.
' Bit2  - On=Append to Existing Map, Off=Don't.
' Bit3-4  Not Used
' Bit5-8 (MapStyle)
'       0 - Town Map
'       1 - Wilderness Map
'       2 - Building Map
'       3 - Dungeon Map
' Bit9-16 Not Used
Private myGenerateUponEntry As Integer
' ReGen: Default Size of Map (Max: 256x256)
Private myDefaultWidth As Integer
Private myDefaultHeight As Integer
' ReGen: Default number of Encounters in Map
Private myDefaultEncounters As Integer

Private myRunes(19) As Integer

' myWidth and myHeight are in Tiles. Max is a reasonable number.
Private myWidth As Integer
Private myHeight As Integer
' myLeft and myTop are the last displayed Left and Top of the Map.
Private myLeft As Integer
Private myTop As Integer
' Layers myBottomTile and myTopTile point to tile in Tile graphic (bmp)
Private myTopTile() As Integer
Private myMiddleTile() As Integer
Private myBottomTile() As Integer
' Layer which points to an Encounter.Index. This means a single tile
' points to zero or one Encounter.
Private myEncPointer() As Integer
' myVisible is set of 8 bit flags per coordinate:
'   &H1  Bit 0  - On = Is Hidden, Off = Is Not Hidden
'   &H2  Bit 1  - Not Used
'   &H4  Bit 2  - Not Used
'   &H8  Bit 3  - On = Flip MiddleTile, Off = Don't flip
'   &H10 Bit 4  - On = Flip BottomTile, Off = Don't flip
'   &H20 Bit 5  - On = Flip TopTile, Off = Don't flip
'   &H40 Bit 6  - Not Used
'   &H80 Bit 7  - Not Used
Private myVisible() As Byte

' List of EntryPoints (x,y) coordinates, Plot, Map and EntryPoint Ref)
Private myEntryPoints As New Collection
' Collection of Tiles (picture, swap tile, triggers) for this Map
Private myTiles As New Collection
' Set of TileSet names for the TileSet for this map. Just used as
' a collection point for tiles.
Private myTileSets As New Collection
' myEncounters holds Encounters: Creatures, Items and Triggers.
Private myEncounters As New Collection
' myThemes holds Encounters, PlotItems, Creatures, Items, Triggers and
' Descriptions for random placement
Private myThemes As New Collection

Public Property Get TotalSize() As Long
    TotalSize = myWidth * myHeight
End Property

Public Property Get Pic() As Integer
    Pic = myPic
End Property

Public Property Let Pic(NewValue As Integer)
    myPic = NewValue
End Property

Public Property Get PictureFile() As String
    PictureFile = Trim(myPictureFile)
End Property

Public Property Let PictureFile(NewName As String)
    myPictureFile = Trim(NewName)
End Property

Public Property Get DefaultStyle() As Byte
    DefaultStyle = myDefaultStyle
End Property

Public Property Let DefaultStyle(NewValue As Byte)
    myDefaultStyle = NewValue
End Property

Public Property Get ExperiencePoints() As Integer
    ExperiencePoints = myExperiencePoints
End Property

Public Property Let ExperiencePoints(NewValue As Integer)
    myExperiencePoints = NewValue
End Property

Public Property Get GenerateUponEntry() As Integer
    GenerateUponEntry = (myGenerateUponEntry And &H1) > 0
End Property

Public Property Let GenerateUponEntry(NewValue As Integer)
    If NewValue Then
        myGenerateUponEntry = myGenerateUponEntry Or &H1
    Else
        myGenerateUponEntry = myGenerateUponEntry And (Not &H1)
    End If
End Property

Public Property Get GenerateAppend() As Integer
    GenerateAppend = (myGenerateUponEntry And &H2) > 0
End Property

Public Property Let GenerateAppend(NewValue As Integer)
    If NewValue Then
        myGenerateUponEntry = myGenerateUponEntry Or &H2
    Else
        myGenerateUponEntry = myGenerateUponEntry And (Not &H2)
    End If
End Property

Public Property Get IsOutside() As Integer
    Select Case Me.MapStyle
        Case 0  ' Town
            IsOutside = True
        Case 1  ' Wilderness
            IsOutside = True
        Case Else
            IsOutside = False
    End Select
End Property

Public Property Get MapStyle() As Integer
    MapStyle = CInt((myGenerateUponEntry And &HF0) / 16)
End Property

Public Property Let MapStyle(NewValue As Integer)
    myGenerateUponEntry = myGenerateUponEntry And &HFF0F
    myGenerateUponEntry = myGenerateUponEntry Or (NewValue * 16 And &HF0)
End Property

Public Property Get PartySize() As Integer
    PartySize = CInt(myStyle And &H3)
End Property

Public Property Let PartySize(NewValue As Integer)
    myStyle = myStyle And &HFFFC
    myStyle = myStyle Or (NewValue And &H3)
End Property

Public Property Get PartyAvgLevel() As Integer
    PartyAvgLevel = CInt((myStyle And &HC) / 4)
End Property

Public Property Let PartyAvgLevel(NewValue As Integer)
    myStyle = myStyle And &HFFF3
    myStyle = myStyle Or (NewValue * 4 And &HC)
End Property

Public Property Get IsNoTiles() As Integer
    IsNoTiles = (myStyle And &H20) > 0
End Property

Public Property Let IsNoTiles(NewValue As Integer)
    If NewValue Then
        myStyle = myStyle Or &H20
    Else
        myStyle = myStyle And (Not &H20)
    End If
End Property

Public Property Get IsNoRunes() As Integer
    IsNoRunes = (myStyle And &H40) > 0
End Property

Public Property Let IsNoRunes(NewValue As Integer)
    If NewValue Then
        myStyle = myStyle Or &H40
    Else
        myStyle = myStyle And (Not &H40)
    End If
End Property

Public Property Get EncounterPoints() As Integer
    Dim EncounterX As Encounter
    For Each EncounterX In myEncounters
        EncounterPoints = EncounterPoints + EncounterX.EncounterPoints
    Next EncounterX
End Property

Public Property Get DefaultWidth() As Integer
    DefaultWidth = myDefaultWidth
End Property

Public Property Let DefaultWidth(NewValue As Integer)
    myDefaultWidth = NewValue
End Property

Public Property Get DefaultHeight() As Integer
    DefaultHeight = myDefaultHeight
End Property

Public Property Let DefaultHeight(NewValue As Integer)
    myDefaultHeight = NewValue
End Property

Public Property Get DefaultEncounters() As Integer
    DefaultEncounters = myDefaultEncounters
End Property

Public Property Let DefaultEncounters(NewValue As Integer)
    myDefaultEncounters = NewValue
End Property

Public Property Get Runes(Index As Integer) As Integer
    Runes = myRunes(Index)
End Property

Public Property Let Runes(Index As Integer, NewValue As Integer)
    myRunes(Index) = NewValue
End Property

Public Property Get Encounters() As Collection
    Set Encounters = myEncounters
End Property

Public Property Get EntryPoints() As Collection
    Set EntryPoints = myEntryPoints
End Property

Public Property Set EntryPoints(NewValue As Collection)
    Set myEntryPoints = NewValue
End Property

Public Property Get Tiles() As Collection
    Set Tiles = myTiles
End Property

Public Property Get Index() As Integer
    Index = myIndex
End Property

Public Property Let Index(NewIndex As Integer)
    myIndex = NewIndex
End Property

Public Function AddEntryPoint() As EntryPoint
    ' Adds an EntryPoint and returns a reference to that EntryPoint
    Dim c As Integer
    Dim EntryPointX As EntryPoint
    ' Find new available unused index idenifier
    c = 1
    For Each EntryPointX In myEntryPoints
        If EntryPointX.Index >= c Then
            c = EntryPointX.Index + 1
        End If
    Next EntryPointX
    ' Set the index and add the door. Return the new door's index.
    Set EntryPointX = New EntryPoint
    EntryPointX.Index = c
    EntryPointX.Name = "EntryPoint" & c
    myEntryPoints.Add Item:=EntryPointX, Key:="P" & EntryPointX.Index
    Set AddEntryPoint = EntryPointX
End Function

Public Function AddEntryPointAsIndex(Index As Integer) As EntryPoint
    ' Adds an EntryPoint and returns a reference to that EntryPoint
    Dim c As Integer
    Dim EntryPointX As EntryPoint
    ' Set the index and add the door. Return the new door's index.
    Set EntryPointX = New EntryPoint
    EntryPointX.Index = Index
    EntryPointX.Name = "EntryPoint" & Index
    myEntryPoints.Add Item:=EntryPointX, Key:="P" & EntryPointX.Index
    Set AddEntryPointAsIndex = EntryPointX
End Function

Public Sub RemoveEntryPoint(DeleteKey As String)
    ' Remove Encounter
    myEntryPoints.Remove DeleteKey
End Sub
    
Public Function AddEncounter() As Encounter
    ' Adds an Encounter and returns a reference to that Encounter
    Dim c As Integer
    Dim EncounterX As Encounter
    ' Find new available unused index idenifier
    c = 1
    For Each EncounterX In myEncounters
        If EncounterX.Index >= c Then
            c = EncounterX.Index + 1
        End If
    Next EncounterX
    ' Set the index and add the door. Return the new door's index.
    Set EncounterX = New Encounter
    EncounterX.Index = c
    EncounterX.Name = "Encounter" & c
    myEncounters.Add Item:=EncounterX, Key:="E" & EncounterX.Index
    Set AddEncounter = EncounterX
End Function

Public Function AddEncounterAsIndex(Index As Integer) As Encounter
    ' Adds an Encounter and returns a reference to that Encounter
    Dim c As Integer
    Dim EncounterX As Encounter
    ' Set the index and add the door. Return the new door's index.
    Set EncounterX = New Encounter
    EncounterX.Index = Index
    EncounterX.Name = "Encounter" & Index
    myEncounters.Add Item:=EncounterX, Key:="E" & EncounterX.Index
    Set AddEncounterAsIndex = EncounterX
End Function

Public Function AddTheme() As Theme
    ' Adds an Theme and returns a reference to that Theme
    Dim c As Integer
    Dim ThemeX As Theme
    ' Find new available unused index idenifier
    c = 1
    For Each ThemeX In myThemes
        If ThemeX.Index >= c Then
            c = ThemeX.Index + 1
        End If
    Next ThemeX
    ' Set the index and add the Theme. Return the new Theme's index.
    Set ThemeX = New Theme
    ThemeX.Index = c
    ThemeX.Name = "Theme" & c
    myThemes.Add Item:=ThemeX, Key:="R" & ThemeX.Index
    Set AddTheme = ThemeX
End Function

Public Function AddThemeAsIndex(Index As Integer) As Theme
    ' Adds an Theme and returns a reference to that Theme
    Dim c As Integer
    Dim ThemeX As Theme
    ' Set the index and add the Theme. Return the new Theme's index.
    Set ThemeX = New Theme
    ThemeX.Index = Index
    ThemeX.Name = "Theme" & Index
    myThemes.Add Item:=ThemeX, Key:="R" & ThemeX.Index
    Set AddThemeAsIndex = ThemeX
End Function

Public Function AddTile() As Tile
    ' Adds a Tile and returns a reference to that Tile
    Dim c As Integer
    Dim TileX As Tile
    ' Find new available unused index idenifier
    c = 1
    For Each TileX In myTiles
        If TileX.Index >= c Then
            c = TileX.Index + 1
        End If
    Next TileX
    ' Create the Tile and set the Index
    Set TileX = New Tile
    TileX.Index = c
    TileX.Name = "Tile" & c
    myTiles.Add Item:=TileX, Key:="L" & TileX.Index
    Set AddTile = TileX
End Function

Public Function AddTileAsIndex(Index As Integer) As Tile
    ' Adds a Tile and returns a reference to that Tile
    Dim c As Integer
    Dim TileX As Tile
    ' Create the Tile and set the Index
    Set TileX = New Tile
    TileX.Index = Index
    TileX.Name = "Tile" & Index
    myTiles.Add Item:=TileX, Key:="L" & TileX.Index
    Set AddTileAsIndex = TileX
End Function


Public Sub RemoveEncounter(DeleteKey As String)
    Dim c As Integer, x As Integer, y As Integer
    ' Remove all references to this Encounter
    c = myEncounters(DeleteKey).Index
    For x = 0 To myWidth: For y = 0 To myHeight
        If myEncPointer(x, y) = c Then
            myEncPointer(x, y) = 0
        End If
    Next y: Next x
    ' Remove Encounter
    myEncounters.Remove DeleteKey
End Sub

Public Sub RemoveTile(DeleteKey As String)
    Dim Index As Integer, x As Integer, y As Integer
    Dim TileX As Tile
    ' Remove references to this tile
    Index = myTiles(DeleteKey).Index
    For x = 0 To myWidth
        For y = 0 To myHeight
            If myTopTile(x, y) = Index Then
                myTopTile(x, y) = 0
            End If
            If myMiddleTile(x, y) = Index Then
                myMiddleTile(x, y) = 0
            End If
            If myBottomTile(x, y) = Index Then
                myBottomTile(x, y) = 0
            End If
        Next y
    Next x
    For Each TileX In myTiles
        If TileX.SwapTile = Index Then
            TileX.SwapTile = 0
        End If
    Next TileX
    ' Remove the tile
    myTiles.Remove DeleteKey
End Sub

Public Sub RemoveTheme(DeleteKey As String)
    myThemes.Remove DeleteKey
End Sub

Public Property Get TileSets() As Collection
    Set TileSets = myTileSets
End Property

Public Property Get Themes() As Collection
    Set Themes = myThemes
End Property

Public Function AddTileSet() As TileSet
    ' Adds a TileSet and returns a reference to that TileSet
    Dim c As Integer
    Dim TileSetX As TileSet
    ' Find new available unused index idenifier
    c = 1
    For Each TileSetX In myTileSets
        If TileSetX.Index <> c Then
            Exit For
        End If
        c = c + 1
    Next TileSetX
    ' Create the TileSet and set the Index
    Set TileSetX = New TileSet
    TileSetX.Index = c
    TileSetX.Name = "TileSet" & c
    myTileSets.Add Item:=TileSetX, Key:="W" & TileSetX.Index
    Set AddTileSet = TileSetX
End Function

Public Sub RemoveTileSet(DeleteKey As String)
    Dim c As Integer, TileX As Tile
    ' Remove pointer from all Tiles
    c = myTileSets(DeleteKey).Index
    For Each TileX In myTiles
        If TileX.TileSet = c Then
            TileX.TileSet = 0
        End If
    Next TileX
    ' Remove TileSet
    myTileSets.Remove DeleteKey
End Sub

Public Property Get Width() As Integer
    Width = myWidth
End Property

Public Property Let Width(x As Integer)
    ReDim TempB(myWidth, myHeight) As Byte
    ReDim TempM(myWidth, myHeight) As Byte
    ReDim TempT(myWidth, myHeight) As Byte
    ReDim TempV(myWidth, myHeight) As Byte
    ReDim TempE(myWidth, myHeight) As Integer
    Dim c As Integer, i As Integer
    ' Only change width if truly different
    If x <> myWidth Then
        If x < 0 Then
            x = 0
        End If
        ' Preserve Arrays
        For c = 0 To myWidth: For i = 0 To myHeight
            TempB(c, i) = myBottomTile(c, i)
            TempM(c, i) = myMiddleTile(c, i)
            TempT(c, i) = myTopTile(c, i)
            TempV(c, i) = myVisible(c, i)
            TempE(c, i) = myEncPointer(c, i)
        Next i: Next c
        ' Resize
        ReDim myBottomTile(x, myHeight) As Integer
        ReDim myMiddleTile(x, myHeight) As Integer
        ReDim myTopTile(x, myHeight) As Integer
        ReDim myVisible(x, myHeight) As Byte
        ReDim myEncPointer(x, myHeight) As Integer
        ' Copy back
        For c = 0 To Least(myWidth, x): For i = 0 To myHeight
            myBottomTile(c, i) = TempB(c, i)
            myMiddleTile(c, i) = TempM(c, i)
            myTopTile(c, i) = TempT(c, i)
            myVisible(c, i) = TempV(c, i)
            myEncPointer(c, i) = TempE(c, i)
        Next i: Next c
        myWidth = x
    End If
End Property

Public Property Get Height() As Integer
    Height = myHeight
End Property

Public Property Let Height(y As Integer)
    ReDim TempB(myWidth, myHeight) As Byte
    ReDim TempM(myWidth, myHeight) As Byte
    ReDim TempT(myWidth, myHeight) As Byte
    ReDim TempV(myWidth, myHeight) As Byte
    ReDim TempE(myWidth, myHeight) As Integer
    Dim c As Integer, i As Integer
    ' Only change height if truly different
    If y <> myHeight Then
        If y < 0 Then
            y = 0
        End If
        ' Preserve Arrays
        For c = 0 To myWidth: For i = 0 To myHeight
            TempB(c, i) = myBottomTile(c, i)
            TempM(c, i) = myMiddleTile(c, i)
            TempT(c, i) = myTopTile(c, i)
            TempV(c, i) = myVisible(c, i)
            TempE(c, i) = myEncPointer(c, i)
        Next i: Next c
        ' Resize
        ReDim myBottomTile(myWidth, y) As Integer
        ReDim myMiddleTile(myWidth, y) As Integer
        ReDim myTopTile(myWidth, y) As Integer
        ReDim myVisible(myWidth, y) As Byte
        ReDim myEncPointer(myWidth, y) As Integer
        ' Copy back
        For c = 0 To myWidth: For i = 0 To Least(myHeight, y)
            myBottomTile(c, i) = TempB(c, i)
            myMiddleTile(c, i) = TempM(c, i)
            myTopTile(c, i) = TempT(c, i)
            myVisible(c, i) = TempV(c, i)
            myEncPointer(c, i) = TempE(c, i)
        Next i: Next c
        myHeight = y
    End If
End Property

Public Property Get Left() As Integer
    Left = myLeft
End Property

Public Property Let Left(NewLeft As Integer)
    myLeft = NewLeft
End Property

Public Property Get Top() As Integer
    Top = myTop
End Property

Public Property Let Top(NewTop As Integer)
    myTop = NewTop
End Property

Public Sub ReadFromFileHeader(FromFile As Integer)
    Dim c As Integer, Cnt As Long
    ' Read Map Name and descriptors
    Get FromFile, , myVersion
    Get FromFile, , Cnt
    myName = ""
    For c = 1 To Cnt
        myName = myName & " "
    Next c
    Get FromFile, , myName
    Get FromFile, , myIndex
End Sub

Public Sub ReadFromFile(FromFile As Integer)
    Dim c As Integer, x As Integer, y As Integer, Cnt As Long
    Dim TileX As Tile, EncounterX As Encounter
    Dim TileSetX As TileSet, EntryPointX As EntryPoint, ThemeX As Theme
    ' Read File Header
    ReadFromFileHeader FromFile
    ' Read other parameters
    Get FromFile, , Cnt
    myComments = ""
    For c = 1 To Cnt
        myComments = myComments & " "
    Next c
    Get FromFile, , myComments
    ' Read PictureFile for Map
    Get FromFile, , Cnt
    myPictureFile = ""
    For c = 1 To Cnt
        myPictureFile = myPictureFile & " "
    Next c
    Get FromFile, , myPictureFile
    ' Read Map Layers
    Get FromFile, , myWidth
    Get FromFile, , myHeight
    Get FromFile, , myLeft
    Get FromFile, , myTop
    ReDim myBottomTile(myWidth, myHeight) As Integer
    ReDim myMiddleTile(myWidth, myHeight) As Integer
    ReDim myTopTile(myWidth, myHeight) As Integer
    ReDim myVisible(myWidth, myHeight) As Byte
    ReDim myEncPointer(myWidth, myHeight) As Integer
    For x = 0 To myWidth: For y = 0 To myHeight
        Get FromFile, , myBottomTile(x, y)
        Get FromFile, , myMiddleTile(x, y)
        Get FromFile, , myTopTile(x, y)
        Get FromFile, , myVisible(x, y)
        Get FromFile, , myEncPointer(x, y)
    Next y: Next x
    ' Read ReGen Parameters
    Get FromFile, , myDefaultStyle
    Get FromFile, , myExperiencePoints
    Get FromFile, , myStyle
    Get FromFile, , myGenerateUponEntry
    Get FromFile, , myDefaultWidth
    Get FromFile, , myDefaultHeight
    Get FromFile, , myDefaultEncounters
    ' Read Runes
    For c = 0 To 19
        Get FromFile, , myRunes(c)
    Next c
    ' Read EntryPoints
    Get FromFile, , Cnt
    For c = 1 To Cnt
        Set EntryPointX = New EntryPoint
        EntryPointX.ReadFromFile FromFile
        myEntryPoints.Add Item:=EntryPointX, Key:="P" & EntryPointX.Index
    Next c
    ' Read TileSets
    Get FromFile, , Cnt
    For c = 1 To Cnt
        Set TileSetX = New TileSet
        TileSetX.ReadFromFile FromFile
        myTileSets.Add Item:=TileSetX, Key:="W" & TileSetX.Index
    Next c
    ' Read Tiles
    Get FromFile, , Cnt
    For c = 1 To Cnt
        Set TileX = New Tile
        TileX.ReadFromFile FromFile
        myTiles.Add Item:=TileX, Key:="L" & TileX.Index
    Next c
    ' Read Themes
    Get FromFile, , Cnt
    For c = 1 To Cnt
        Set ThemeX = New Theme
        ThemeX.ReadFromFile FromFile
        myThemes.Add Item:=ThemeX, Key:="R" & ThemeX.Index
    Next c
    ' Read Encounters
    Get FromFile, , Cnt
    For c = 1 To Cnt
        Set EncounterX = New Encounter
        EncounterX.ReadFromFile FromFile
        myEncounters.Add Item:=EncounterX, Key:="E" & EncounterX.Index
    Next
End Sub

Public Sub Copy(FromMap As Map)
    Dim c As Integer, x As Integer, y As Integer
    Dim TileX As Tile, EncounterX As Encounter
    Dim TileSetX As TileSet, ThemeX As Theme, EntryPointX As EntryPoint
    ' Copy Map Name and descriptors
    myName = FromMap.Name
    myComments = FromMap.Comments
    ' Copy PictureFile for Map
    myPictureFile = FromMap.PictureFile
    ' Copy Map Layers
    Me.Width = FromMap.Width
    Me.Height = FromMap.Height
    myLeft = FromMap.Left
    myTop = FromMap.Top
    For x = 0 To myWidth: For y = 0 To myHeight
        myBottomTile(x, y) = FromMap.BottomTile(x, y)
        myMiddleTile(x, y) = FromMap.MiddleTile(x, y)
        myTopTile(x, y) = FromMap.TopTile(x, y)
        myVisible(x, y) = FromMap.Visible(x, y)
        myEncPointer(x, y) = FromMap.EncPointer(x, y)
    Next y: Next x
    ' Copy ReGen Parameters
    myDefaultStyle = FromMap.DefaultStyle
    myExperiencePoints = FromMap.ExperiencePoints
    Me.GenerateUponEntry = FromMap.GenerateUponEntry
    Me.MapStyle = FromMap.MapStyle
    myDefaultWidth = FromMap.DefaultWidth
    myDefaultHeight = FromMap.DefaultHeight
    myDefaultEncounters = FromMap.DefaultEncounters
    ' Copy Runes
    For c = 0 To 19
        Me.Runes(c) = FromMap.Runes(c)
    Next c
    ' Copy TileSets
    Set myTileSets = New Collection
    For c = 1 To FromMap.TileSets.Count
        Set TileSetX = Me.AddTileSet
        TileSetX.Copy FromMap.TileSets(c)
    Next c
    ' Copy Tiles
    Set myTiles = New Collection
    For c = 1 To FromMap.Tiles.Count
        Set TileX = Me.AddTileAsIndex(FromMap.Tiles(c).Index)
        TileX.Copy FromMap.Tiles(c)
    Next c
    ' Copy Themes
    Set myThemes = New Collection
    For c = 1 To FromMap.Themes.Count
        Set ThemeX = Me.AddThemeAsIndex(FromMap.Themes(c).Index)
        ThemeX.Copy FromMap.Themes(c)
    Next c
    ' Copy Encounters
    Set myEncounters = New Collection
    For c = 1 To FromMap.Encounters.Count
        Set EncounterX = Me.AddEncounterAsIndex(FromMap.Encounters(c).Index)
        EncounterX.Copy FromMap.Encounters(c)
    Next c
    ' Copy EntryPoints
    Set myEntryPoints = New Collection
    For c = 1 To FromMap.EntryPoints.Count
        Set EntryPointX = Me.AddEntryPointAsIndex(FromMap.EntryPoints(c).Index)
        EntryPointX.Copy FromMap.EntryPoints(c)
    Next c
End Sub

Public Sub SaveToFile(ToFile As Integer)
    Dim c As Integer, x As Integer, y As Integer
    ' Save Map Name and descriptors
    Put ToFile, , myVersion
    Put ToFile, , Len(myName)
    Put ToFile, , myName
    Put ToFile, , myIndex
    Put ToFile, , Len(myComments)
    Put ToFile, , myComments
    ' Save PictureFile for Map
    Put ToFile, , Len(myPictureFile)
    Put ToFile, , myPictureFile
    ' Save Map Layers
    Put ToFile, , myWidth
    Put ToFile, , myHeight
    Put ToFile, , myLeft
    Put ToFile, , myTop
    For x = 0 To myWidth: For y = 0 To myHeight
        Put ToFile, , myBottomTile(x, y)
        Put ToFile, , myMiddleTile(x, y)
        Put ToFile, , myTopTile(x, y)
        Put ToFile, , myVisible(x, y)
        Put ToFile, , myEncPointer(x, y)
    Next y: Next x
    ' Save ReGen Parameters
    Put ToFile, , myDefaultStyle
    Put ToFile, , myExperiencePoints
    Put ToFile, , myStyle
    Put ToFile, , myGenerateUponEntry
    Put ToFile, , myDefaultWidth
    Put ToFile, , myDefaultHeight
    Put ToFile, , myDefaultEncounters
    ' Save Runes
    For c = 0 To 19
        Put ToFile, , myRunes(c)
    Next c
    ' Save EntryPoints
    Put ToFile, , myEntryPoints.Count
    For c = 1 To myEntryPoints.Count
        myEntryPoints(c).SaveToFile ToFile
    Next c
    ' Save TileSets
    Put ToFile, , myTileSets.Count
    For c = 1 To myTileSets.Count
        myTileSets(c).SaveToFile ToFile
    Next c
    ' Save Tiles
    Put ToFile, , myTiles.Count
    For c = 1 To myTiles.Count
        myTiles(c).SaveToFile ToFile
    Next c
    ' Save Themes
    Put ToFile, , myThemes.Count
    For c = 1 To myThemes.Count
        myThemes(c).SaveToFile ToFile
    Next
    ' Save Encounters
    Put ToFile, , myEncounters.Count
    For c = 1 To myEncounters.Count
        myEncounters(c).SaveToFile ToFile
    Next
End Sub

Public Property Get Hidden(ByVal x As Integer, ByVal y As Integer) As Integer
    If x < 0 Or y < 0 Or x > myWidth Or y > myHeight Then
        Hidden = False
    Else
        Hidden = (myVisible(x, y) And &H1) > 0
    End If
End Property

Public Property Let Hidden(ByVal x As Integer, ByVal y As Integer, NewValue As Integer)
    If x < 0 Or y < 0 Or x > myWidth Or y > myHeight Then
        Exit Property
    Else
        If NewValue Then
            myVisible(x, y) = myVisible(x, y) Or &H1
        Else
            myVisible(x, y) = myVisible(x, y) And (Not &H1)
        End If
    End If
End Property

Public Property Get BottomFlip(ByVal x As Integer, ByVal y As Integer) As Integer
    ' TRUE if flipped at (x,y), FALSE if not
    BottomFlip = (myVisible(x, y) And &H10) > 0
End Property

Public Property Let BottomFlip(ByVal x As Integer, ByVal y As Integer, NewValue As Integer)
    If NewValue Then
        myVisible(x, y) = myVisible(x, y) Or &H10
    Else
        myVisible(x, y) = myVisible(x, y) And (Not &H10)
    End If
End Property

Public Property Get TopFlip(ByVal x As Integer, ByVal y As Integer) As Integer
    ' TRUE if flipped at (x,y), FALSE if not
    TopFlip = (myVisible(x, y) And &H20) > 0
End Property

Public Property Let TopFlip(ByVal x As Integer, ByVal y As Integer, NewValue As Integer)
    If NewValue Then
        myVisible(x, y) = myVisible(x, y) Or &H20
    Else
        myVisible(x, y) = myVisible(x, y) And (Not &H20)
    End If
End Property

Public Property Get Tile(ByVal Index As Integer, ByVal x As Integer, ByVal y As Integer) As Integer
    Select Case Index
        Case 0
            Tile = myBottomTile(x, y)
        Case 1
            Tile = myMiddleTile(x, y)
        Case 2
            Tile = myTopTile(x, y)
    End Select
End Property

Public Property Let Tile(ByVal Index As Integer, ByVal x As Integer, ByVal y As Integer, NewValue As Integer)
    Select Case Index
        Case 0
            myBottomTile(x, y) = NewValue
        Case 1
            myMiddleTile(x, y) = NewValue
        Case 2
            myTopTile(x, y) = NewValue
    End Select
End Property

Public Property Get BottomTile(ByVal x As Integer, ByVal y As Integer) As Integer
    BottomTile = myBottomTile(x, y)
End Property

Public Property Let BottomTile(ByVal x As Integer, ByVal y As Integer, NewValue As Integer)
    myBottomTile(x, y) = NewValue
End Property

Public Property Get TopTile(ByVal x As Integer, ByVal y As Integer) As Integer
    TopTile = myTopTile(x, y)
End Property

Public Property Let TopTile(ByVal x As Integer, ByVal y As Integer, NewValue As Integer)
    myTopTile(x, y) = NewValue
End Property

Public Property Get MiddleTile(ByVal x As Integer, ByVal y As Integer) As Integer
    MiddleTile = myMiddleTile(x, y)
End Property

Public Property Let MiddleTile(ByVal x As Integer, ByVal y As Integer, NewValue As Integer)
    myMiddleTile(x, y) = NewValue
End Property

Public Property Get MiddleFlip(ByVal x As Integer, ByVal y As Integer) As Integer
    ' TRUE if flipped at (x,y), FALSE if not
    MiddleFlip = (myVisible(x, y) And &H8) > 0
End Property

Public Property Let MiddleFlip(ByVal x As Integer, ByVal y As Integer, NewValue As Integer)
    If NewValue Then
        myVisible(x, y) = myVisible(x, y) Or &H8
    Else
        myVisible(x, y) = myVisible(x, y) And (Not &H8)
    End If
End Property

Public Function MovementCost(ByVal x As Integer, ByVal y As Integer) As Integer
    Dim c As Integer, i As Integer
    ' Calculates the average cost in 1/10 turn units of moving through a given tile
    ' Ranges from 1 to 10000.
    If x < 0 Or x > myWidth Or y < 0 Or y > myHeight Then
        c = 0
    Else
        c = 0: i = 0
        If myTopTile(x, y) > 0 Then
            c = c + myTiles("L" & myTopTile(x, y)).MovementCostTurns
            i = i + 1
        End If
        If myMiddleTile(x, y) > 0 Then
            c = c + myTiles("L" & myMiddleTile(x, y)).MovementCostTurns
            i = i + 1
        End If
        If myBottomTile(x, y) > 0 Then
            c = c + myTiles("L" & myBottomTile(x, y)).MovementCostTurns
            i = i + 1
        End If
        If i > 0 Then
            c = Int(c / i)
        End If
    End If
    MovementCost = c
End Function

Public Function Blocked(ByVal x As Integer, ByVal y As Integer, ByVal Side) As Integer
    Dim c As Integer, i As Integer
    ' Tile is blocked by any in Top, Middle or Bottom on either side
    c = 0
    If myTopTile(x, y) > 0 Then
        c = c + myTiles("L" & myTopTile(x, y)).Blocked(Side - (1 - 2 * (Side Mod 2)) * Me.TopFlip(x, y))
    End If
    If myMiddleTile(x, y) > 0 Then
        c = c + myTiles("L" & myMiddleTile(x, y)).Blocked(Side - (1 - 2 * (Side Mod 2)) * Me.MiddleFlip(x, y))
    End If
    If myBottomTile(x, y) > 0 Then
        c = c + myTiles("L" & myBottomTile(x, y)).Blocked(Side - (1 - 2 * (Side Mod 2)) * Me.BottomFlip(x, y))
    End If
    ' Check surrounding tile
    Select Case Side
        Case 0  ' Top Left
            y = y - 1: Side = 3
        Case 1  ' Top Right
            x = x + 1: Side = 2
        Case 2  ' Bottom Left
            x = x - 1: Side = 1
        Case 3  ' Bottom Right
            y = y + 1: Side = 0
        Case 4  ' Up
            y = y - 1: x = x + 1: Side = 7
        Case 5  ' Left
            y = y - 1: x = x - 1: Side = 6
        Case 6  ' Right
            y = y + 1: x = x + 1: Side = 5
        Case 7  ' Down
            y = y + 1: x = x - 1: Side = 4
    End Select
    If x < 0 Or x > myWidth Or y < 0 Or y > myHeight Then
        c = -1
    ElseIf myTopTile(x, y) + myMiddleTile(x, y) + myBottomTile(x, y) = 0 Then
        c = -1
    Else
        If myTopTile(x, y) > 0 Then
            c = c + myTiles("L" & myTopTile(x, y)).Blocked(Side - (1 - 2 * (Side Mod 2)) * Me.TopFlip(x, y))
        End If
        If myMiddleTile(x, y) > 0 Then
            c = c + myTiles("L" & myMiddleTile(x, y)).Blocked(Side - (1 - 2 * (Side Mod 2)) * Me.MiddleFlip(x, y))
        End If
        If myBottomTile(x, y) > 0 Then
            c = c + myTiles("L" & myBottomTile(x, y)).Blocked(Side - (1 - 2 * (Side Mod 2)) * Me.BottomFlip(x, y))
        End If
    End If
    Blocked = (c < 0)
End Function

Public Function See(ByVal x As Integer, ByVal y As Integer, ByVal Side) As Integer
    Dim c As Integer, i As Integer
    ' Tile is See by any in Top, Middle or Bottom on either side
    c = 0
    If myTopTile(x, y) > 0 Then
        c = c + myTiles("L" & myTopTile(x, y)).See(Side - (1 - 2 * (Side Mod 2)) * Me.TopFlip(x, y))
    End If
    If myMiddleTile(x, y) > 0 Then
        c = c + myTiles("L" & myMiddleTile(x, y)).See(Side - (1 - 2 * (Side Mod 2)) * Me.MiddleFlip(x, y))
    End If
    If myBottomTile(x, y) > 0 Then
        c = c + myTiles("L" & myBottomTile(x, y)).See(Side - (1 - 2 * (Side Mod 2)) * Me.BottomFlip(x, y))
    End If
    ' Check surrounding tile
    Select Case Side
        Case 0  ' Top Left
            y = y - 1: Side = 3
        Case 1  ' Top Right
            x = x + 1: Side = 2
        Case 2  ' Bottom Left
            x = x - 1: Side = 1
        Case 3  ' Bottom Right
            y = y + 1: Side = 0
        Case 4  ' Up
            y = y - 1: x = x + 1: Side = 7
        Case 5  ' Left
            y = y - 1: x = x - 1: Side = 6
        Case 6  ' Right
            y = y + 1: x = x + 1: Side = 5
        Case 7  ' Down
            y = y + 1: x = x - 1: Side = 4
    End Select
    If x < 0 Or x > myWidth Or y < 0 Or y > myHeight Then
        c = 0
    Else
        If myTopTile(x, y) > 0 Then
            c = c + myTiles("L" & myTopTile(x, y)).See(Side - (1 - 2 * (Side Mod 2)) * Me.TopFlip(x, y))
        End If
        If myMiddleTile(x, y) > 0 Then
            c = c + myTiles("L" & myMiddleTile(x, y)).See(Side - (1 - 2 * (Side Mod 2)) * Me.MiddleFlip(x, y))
        End If
        If myBottomTile(x, y) > 0 Then
            c = c + myTiles("L" & myBottomTile(x, y)).See(Side - (1 - 2 * (Side Mod 2)) * Me.BottomFlip(x, y))
        End If
    End If
    See = Not (c > -1)
End Function

Public Property Get EncPointer(ByVal x As Integer, ByVal y As Integer) As Integer
    EncPointer = myEncPointer(x, y)
End Property

Public Property Let EncPointer(ByVal x As Integer, ByVal y As Integer, NewValue As Integer)
    myEncPointer(x, y) = NewValue
End Property

Public Property Get Visible(ByVal x As Integer, ByVal y As Integer) As Integer
    Visible = myVisible(x, y)
End Property

Public Property Let Visible(ByVal x As Integer, ByVal y As Integer, NewValue As Integer)
    myVisible(x, y) = NewValue
End Property

Public Property Get Name() As String
    Name = Trim(myName)
End Property

Public Property Let Name(NewName As String)
    myName = Trim(NewName)
End Property

Public Property Get Comments() As String
    Comments = Trim(myComments)
End Property

Public Property Let Comments(NewComments As String)
    myComments = Trim(NewComments)
End Property

Private Sub Class_Initialize()
    Dim c As Integer, i As Integer
    ReDim myBottomTile(31, 31) As Integer
    ReDim myMiddleTile(31, 31) As Integer
    ReDim myTopTile(31, 31) As Integer
    ReDim myVisible(31, 31) As Byte
    ReDim myEncPointer(31, 31) As Integer
    myWidth = 31: myHeight = 31
    For c = 0 To 31: For i = 0 To 31: Me.Hidden(c, i) = True: Next i: Next c
    myDefaultWidth = myWidth: myDefaultHeight = myHeight
    myDefaultEncounters = 30
    myLeft = Int(myWidth / 16) - 1
    myTop = -Int(myWidth / 16) - 2
    myName = "Untitled"
    For c = 0 To 19
        ' Set number of runes
        Select Case c
            Case 0, 4, 8, 12, 16
                myRunes(c) = 1
            Case 1, 2, 5, 6, 9, 10, 13, 14, 17, 18
                myRunes(c) = 2
            Case 3, 7, 11, 15, 19
                myRunes(c) = 3
        End Select
    Next c
    ' Default bmp is bdout2.bmp with 128 tiles
    myPic = -1
    myPictureFile = "bdout2.bmp"
End Sub
