VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "MapSketch"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' Used to control versions
Private myVersion As Integer
' Unique number for this MapSketch inside an UberWizard
Private myIndex As Integer
' Name of the Map it represents
Private myMapName As String
' Comments about the Map it represents
Private myMapComments As String
' FullPath of where the Map is located (Area file, *.rsa)
Private myFullPath As String
' Created for Party Size
' Bit1-2 - 0 = Solo, 1 = 2-3, 2 = 4-6, 3 = 7-9
' With Average Party Level
' Bit3-4 - 0 = 1-3, 1 = 4-6, 2 = 7-10, 3 = 10+
' Bit5-16   Not Used
Dim myStyle As Integer
' Bit1  - On=IsUsed, Off=Not Used
' Bit2  - On=GenerateUponEntry, Off=Not
' Bit3  - On=NeedsThemes, Off=Not
' Bit4  - On=IsMainMap, Off=Not
' Bit5-8 - MapStyle
' Bit9  - On=IsSelected to be used, Off=Not
' Bit10 - On=GenerateThemes, Off=Not
' Bit11-16 Not Used
Private myFlags As Integer
' Number of times it can be used
Private myUses As Integer
' Save settings for a random build of a Map
Private mySize As Integer
' Map Set Size
Private myTotalSize As Long
' If a Tome Map: This is the TomeIndex in UberWiz
Private myTomeIndex As Integer
' Unknown use
Private myEncounterCount As Integer
' Map Index in Area where this MapSketch is located
Private myMapIndexInArea As Integer
' Map Index in Area where this MapSketch is located
Private myAreaIndexInArea As Integer
' Area Index where MapSketch is placed when create Tome
Private myTomeIndexInArea As Integer
' Tome Index where MapSketch is placed when create Tome
Private myAreaIndex As Integer
' Map Index where MapSketch is placed in Area
Private myMapIndex As Integer
' Collection of the EntryPoints in the Map. Connected up as we progress and build.
Private myEntryPoints As New Collection
' Collection of ThemeSketchs (if allowed in this Map)
Private myThemeSketchs As New Collection
' Collection of Themes used as Quest/Major Theme parts
Private myThemeQuests As New Collection

Public Property Get Version() As Integer
    Version = myVersion
End Property

Public Property Let Version(NewValue As Integer)
    myVersion = NewValue
End Property

Public Property Get Index() As Integer
    Index = myIndex
End Property

Public Property Let Index(NewValue As Integer)
    myIndex = NewValue
End Property

Public Property Get Uses() As Integer
    Uses = myUses
End Property

Public Property Let Uses(NewValue As Integer)
    myUses = NewValue
End Property

Public Property Get TomeIndex() As Integer
    TomeIndex = myTomeIndex
End Property

Public Property Let TomeIndex(NewValue As Integer)
    myTomeIndex = NewValue
End Property

Public Property Get Size() As Integer
    Size = mySize
End Property

Public Property Let Size(NewValue As Integer)
    mySize = NewValue
End Property

Public Property Get TotalSize() As Long
    TotalSize = myTotalSize
End Property

Public Property Let TotalSize(NewValue As Long)
    myTotalSize = NewValue
End Property

Public Property Get EncounterCount() As Integer
    EncounterCount = myEncounterCount
End Property

Public Property Let EncounterCount(NewValue As Integer)
    myEncounterCount = NewValue
End Property

Public Property Get Flags() As Byte
    Flags = myFlags
End Property

Public Property Let Flags(NewValue As Byte)
    myFlags = NewValue
End Property

Public Property Get MapName() As String
    MapName = Trim(myMapName)
End Property

Public Property Let MapName(NewMapName As String)
    myMapName = Trim(NewMapName)
End Property

Public Property Get MapComments() As String
    MapComments = Trim(myMapComments)
End Property

Public Property Let MapComments(NewMapComments As String)
    myMapComments = Trim(NewMapComments)
End Property

Public Property Get FullPath() As String
    FullPath = Trim(myFullPath)
End Property

Public Property Let FullPath(NewFullPath As String)
    myFullPath = Trim(NewFullPath)
End Property

Public Property Get IsSelected() As Integer
    IsSelected = (myFlags And &H100) > 0
End Property

Public Property Let IsSelected(NewValue As Integer)
    If NewValue Then
        myFlags = myFlags Or &H100
    Else
        myFlags = myFlags And (Not &H100)
    End If
End Property

Public Property Get IsUsed() As Integer
    IsUsed = (myFlags And &H1) > 0
End Property

Public Property Let IsUsed(NewValue As Integer)
    If NewValue Then
        myFlags = myFlags Or &H1
    Else
        myFlags = myFlags And (Not &H1)
    End If
End Property

Public Property Get GenerateUponEntry() As Integer
    GenerateUponEntry = (myFlags And &H2) > 0
End Property

Public Property Let GenerateUponEntry(NewValue As Integer)
    If NewValue Then
        myFlags = myFlags Or &H2
    Else
        myFlags = myFlags And (Not &H2)
    End If
End Property

Public Property Get GenerateThemes() As Integer
    GenerateThemes = (myFlags And &H200) > 0
End Property

Public Property Let GenerateThemes(NewValue As Integer)
    If NewValue Then
        myFlags = myFlags Or &H200
    Else
        myFlags = myFlags And (Not &H200)
    End If
End Property

Public Property Get NeedsThemes() As Integer
    NeedsThemes = (myFlags And &H4) > 0
End Property

Public Property Let NeedsThemes(NewValue As Integer)
    If NewValue Then
        myFlags = myFlags Or &H4
    Else
        myFlags = myFlags And (Not &H4)
    End If
End Property

Public Property Get IsMainMap() As Integer
    IsMainMap = (myFlags And &H8) > 0
End Property

Public Property Let IsMainMap(NewValue As Integer)
    If NewValue Then
        myFlags = myFlags Or &H8
    Else
        myFlags = myFlags And (Not &H8)
    End If
End Property

Public Property Get MapStyle() As Integer
    MapStyle = CInt((myFlags And &HF0) / 16)
End Property

Public Property Let MapStyle(NewValue As Integer)
    myFlags = myFlags And &HFF0F
    myFlags = myFlags Or (NewValue * 16 And &HF0)
End Property

Public Property Get PartySize() As Integer
    PartySize = CInt(myStyle And &H3)
End Property

Public Property Let PartySize(NewValue As Integer)
    Dim ThemeSketchX As ThemeSketch
    myStyle = myStyle And &HFFFC
    myStyle = myStyle Or (NewValue And &H3)
End Property

Public Property Get PartyAvgLevel() As Integer
    PartyAvgLevel = CInt((myStyle And &HC) / 4)
End Property

Public Property Let PartyAvgLevel(NewValue As Integer)
    Dim ThemeSketchX As ThemeSketch
    myStyle = myStyle And &HFFF3
    myStyle = myStyle Or (NewValue * 4 And &HC)
End Property

Public Sub IncreaseDifficulty()
    ' Max is 3 and 3 in both
    If Me.PartySize = 3 And Me.PartyAvgLevel = 3 Then
        Exit Sub
    End If
    ' Increase PartySize and AvgLevel by 1 degree
    Select Case Me.PartySize * 4 + Me.PartyAvgLevel
        Case 3, 7, 11
            Me.PartySize = Me.PartySize + 1
            Me.PartyAvgLevel = 0
        Case Else
            Me.PartyAvgLevel = Me.PartyAvgLevel + 1
    End Select
End Sub

Public Property Get Difficulty() As Integer
    Dim c As Integer
    Select Case Me.PartyAvgLevel
        Case 0  ' 1-3
            c = 1
        Case 1  ' 4-6
            c = 3
        Case 2  ' 7-10
            c = 6
        Case 3  ' 10+
            c = 12
    End Select
    Select Case Me.PartySize
        Case 0  ' Solo
            Difficulty = 1 * c
        Case 1  ' 2-3
            Difficulty = 3 * c
        Case 2  ' 4-6
            Difficulty = 5 * c
        Case 3  ' 7-9
            Difficulty = 10 * c
    End Select
End Property

Public Property Get AreaIndex() As Integer
    AreaIndex = myAreaIndex
End Property

Public Property Let AreaIndex(NewValue As Integer)
    myAreaIndex = NewValue
End Property

Public Property Get MapIndex() As Integer
    MapIndex = myMapIndex
End Property

Public Property Let MapIndex(NewValue As Integer)
    myMapIndex = NewValue
End Property

Public Property Get TomeIndexInArea() As Integer
    TomeIndexInArea = myTomeIndexInArea
End Property

Public Property Let TomeIndexInArea(NewValue As Integer)
    myTomeIndexInArea = NewValue
End Property

Public Property Get AreaIndexInArea() As Integer
    AreaIndexInArea = myAreaIndexInArea
End Property

Public Property Let AreaIndexInArea(NewValue As Integer)
    myAreaIndexInArea = NewValue
End Property

Public Property Get MapIndexInArea() As Integer
    MapIndexInArea = myMapIndexInArea
End Property

Public Property Let MapIndexInArea(NewValue As Integer)
    myMapIndexInArea = NewValue
End Property

Public Function AddEntryPoint() As EntryPoint
    ' Adds an EntryPoint and returns a reference to that EntryPoint
    Dim c As Integer
    Dim EntryPointX As EntryPoint
    ' Find new available unused index idenifier
    c = 1
    For Each EntryPointX In myEntryPoints
        If EntryPointX.Index >= c Then
            c = EntryPointX.Index + 1
        End If
    Next EntryPointX
    ' Set the index and add the door. Return the new door's index.
    Set EntryPointX = New EntryPoint
    EntryPointX.Index = c
    EntryPointX.Name = "EntryPoint" & c
    myEntryPoints.Add Item:=EntryPointX, Key:="P" & EntryPointX.Index
    Set AddEntryPoint = EntryPointX
End Function

Public Sub RemoveEntryPoint(DeleteKey As String)
    ' Remove EntryPoint
    myEntryPoints.Remove DeleteKey
End Sub

Public Property Get EntryPoints() As Collection
    Set EntryPoints = myEntryPoints
End Property

Public Property Set EntryPoints(NewValue As Collection)
    Set myEntryPoints = NewValue
End Property

Public Property Get ThemeQuests() As Collection
    Set ThemeQuests = myThemeQuests
End Property

Public Function AddThemeQuest() As Theme
    ' Adds an Theme and returns a reference to that Theme
    Dim c As Integer
    Dim ThemeX As Theme
    ' Find new available unused index idenifier
    c = 1
    For Each ThemeX In myThemeQuests
        If ThemeX.Index <> c Then
            Exit For
        End If
        c = c + 1
    Next ThemeX
    ' Set the index and add the Theme. Return the new Theme's index.
    Set ThemeX = New Theme
    ThemeX.Index = c
    ThemeX.Name = "Theme" & c
    myThemeQuests.Add Item:=ThemeX, Key:="R" & ThemeX.Index
    Set AddThemeQuest = ThemeX
End Function

Public Sub RemoveThemeQuest(DeleteKey As String)
    myThemeQuests.Remove DeleteKey
End Sub

Public Function AddThemeSketch() As ThemeSketch
    ' Adds an ThemeSketch and returns a reference to that ThemeSketch
    Dim c As Integer
    Dim ThemeSketchX As ThemeSketch
    ' Find new available unused index idenifier
    c = 1
    For Each ThemeSketchX In myThemeSketchs
        If ThemeSketchX.Index >= c Then
            c = ThemeSketchX.Index + 1
        End If
    Next ThemeSketchX
    ' Set the index and add the door. Return the new door's index.
    Set ThemeSketchX = New ThemeSketch
    ThemeSketchX.Index = c
    ThemeSketchX.Name = "ThemeSketch" & c
    myThemeSketchs.Add Item:=ThemeSketchX, Key:="R" & ThemeSketchX.Index
    Set AddThemeSketch = ThemeSketchX
End Function

Public Sub RemoveThemeSketch(DeleteKey As String)
    ' Remove ThemeSketch
    myThemeSketchs.Remove DeleteKey
End Sub

Public Property Get ThemeSketchs() As Collection
    Set ThemeSketchs = myThemeSketchs
End Property

Public Property Set ThemeSketchs(NewValue As Collection)
    Set myThemeSketchs = NewValue
End Property

Public Sub Copy(FromMapSketch As MapSketch)
    Dim EntryPointX As EntryPoint
    Dim c As Integer
    myVersion = FromMapSketch.Version
    myMapName = FromMapSketch.MapName
    myMapComments = FromMapSketch.MapComments
    myFullPath = FromMapSketch.FullPath
    myFlags = FromMapSketch.Flags
    myTomeIndex = FromMapSketch.TomeIndex
    myAreaIndex = FromMapSketch.AreaIndex
    myMapIndex = FromMapSketch.MapIndex
    ' Copy Entry Points
    For c = 1 To FromMapSketch.EntryPoints.Count
        Set EntryPointX = Me.AddEntryPoint
        EntryPointX.Copy FromMapSketch.EntryPoints(c)
    Next c
End Sub

Public Sub ReadFromFile(FromFile As Integer)
    Dim c As Integer, Cnt As Long
    Dim EntryPointX As EntryPoint
    Get FromFile, , myVersion
    Get FromFile, , myIndex
    Get FromFile, , Cnt
    myMapName = ""
    For c = 1 To Cnt
        myMapName = myMapName & " "
    Next c
    Get FromFile, , myMapName
    Get FromFile, , Cnt
    myMapComments = ""
    For c = 1 To Cnt
        myMapComments = myMapComments & " "
    Next c
    Get FromFile, , myMapComments
    Get FromFile, , Cnt
    myFullPath = ""
    For c = 1 To Cnt
        myFullPath = myFullPath & " "
    Next c
    Get FromFile, , myFullPath
    Get FromFile, , myFlags
    Get FromFile, , myTomeIndex
    Get FromFile, , myAreaIndex
    Get FromFile, , myMapIndex
    ' Read EntryPoints
    Get FromFile, , Cnt
    For c = 1 To Cnt
        Set EntryPointX = New EntryPoint
        EntryPointX.ReadFromFile FromFile
        myEntryPoints.Add Item:=EntryPointX, Key:="P" & EntryPointX.Index
    Next c
End Sub

Public Sub SaveToFile(ToFile As Integer)
    Dim c As Integer
    Put ToFile, , myVersion
    Put ToFile, , myIndex
    Put ToFile, , Len(myMapName)
    Put ToFile, , myMapName
    Put ToFile, , Len(myMapComments)
    Put ToFile, , myMapComments
    Put ToFile, , Len(myFullPath)
    Put ToFile, , myFullPath
    Put ToFile, , myFlags
    Put ToFile, , myTomeIndex
    Put ToFile, , myAreaIndex
    Put ToFile, , myMapIndex
    ' Save EntryPoints
    Put ToFile, , myEntryPoints.Count
    For c = 1 To myEntryPoints.Count
        myEntryPoints(c).SaveToFile ToFile
    Next c
End Sub
