VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Tile"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' Version number of Tile
Private myVersion As Integer
' Name of the Tile
Private myName As String
' Picture Reference
Private myPic As Integer
' Unique number for Tile
Private myIndex As Integer
'   &H1  Bit 0  - On = Block Move Upper Left
'   &H2  Bit 1  - On = Block Move Upper Right
'   &H4  Bit 2  - On = Block Move Lower Left
'   &H8  Bit 3  - On = Block Move Lower Right
'   &H10 Bit 4  - On = Can't See Past Upper Left
'   &H20 Bit 5  - On = Can't See Past Upper Right
'   &H40 Bit 6  - On = Can't See Past Lower Left
'   &H80 Bit 7  - On = Can't See Past Lower Right
Private myBlocked As Byte
' Used in Random Dungeon Maker
' 0 - <None>, 1 - Left Wall, 2 - Top Wall, 3 - Floor
' 4 - Left Arch, 5 - Left Door
Private myStyle As Byte
' Movement Cost per Tile
Private myMovementCost As Byte
' Terrain Byte Index: (to be defined)
Private myTerrainType As Byte
'   Bit 0   - On=Flip, Off=No Flip
'   Bit 1-2 - 00 = Light, 01 = Dark, 10 = Very Dark, 11 = Pitch Black
'   Bit 3-7 - Key bits; all 0's not locked. All 1's, no key will work.
Private myKey As Byte
' Number from 1-255 indicating how likely one click of the mouse will
' cause the tile to switch pictures. This is only applicable if the
' key is not set.
Private myChance As Byte
' Pointer to index of Tile Object in Map.Tiles Collection to Swap to.
' 0 is no tile to swap to.
Private mySwapTile As Integer
' Index of TileSet in parent Map Object. Groups tiles under a single
' name and relates to a specific Combat Wallpaper BMP.
Private myTileSet As Integer
' Not Used
Private myNotUsed As Long

Public Property Get Index() As Integer
    Index = myIndex
End Property

Public Property Let Index(NewIndex As Integer)
    myIndex = NewIndex
End Property

Public Property Get Pic() As Integer
    Pic = myPic
End Property

Public Property Let Pic(NewValue As Integer)
    myPic = NewValue
End Property

Public Property Get Name() As String
    Name = Trim(myName)
End Property

Public Property Let Name(NewValue As String)
    myName = Trim(NewValue)
End Property

Public Property Get Chance() As Integer
    Chance = myChance
End Property

Public Property Let Chance(NewValue As Integer)
    myChance = NewValue
End Property

Public Property Get MovementCost() As Integer
    MovementCost = myMovementCost
End Property

Public Property Get MovementCostTurns() As Integer
    Select Case myMovementCost
        Case 0 To 9
            MovementCostTurns = myMovementCost + 1
        Case 10 To 13
            MovementCostTurns = (myMovementCost - 8) * 10
        Case 14 To 18
            MovementCostTurns = (myMovementCost - 13) * 100
        Case 19
            MovementCostTurns = 1000
        Case 20 To 24
            MovementCostTurns = (myMovementCost - 19) * 200
    End Select
End Property

Public Property Let MovementCost(NewValue As Integer)
    myMovementCost = NewValue
End Property

Public Property Get TerrainType() As Integer
    TerrainType = myTerrainType
End Property

Public Property Let TerrainType(NewValue As Integer)
    myTerrainType = NewValue
End Property

Public Property Get SwapTile() As Integer
    SwapTile = mySwapTile
End Property

Public Property Let SwapTile(NewValue As Integer)
    mySwapTile = NewValue
End Property

Public Sub Copy(FromTile As Tile)
    Dim c As Integer, TriggerX As Trigger
    myName = FromTile.Name
    myPic = FromTile.Pic
    myBlocked = FromTile.BlockedAll
    myStyle = FromTile.Style
    myKey = FromTile.Key
    Me.Flip = FromTile.Flip
    myChance = FromTile.Chance
    mySwapTile = FromTile.SwapTile
    myTileSet = FromTile.TileSet
End Sub

Public Sub SaveToFile(ToFile As Integer)
    Dim c As Integer
    Put ToFile, , myVersion
    Put ToFile, , Len(myName)
    Put ToFile, , myName
    Put ToFile, , myIndex
    Put ToFile, , myPic
    Put ToFile, , myBlocked
    Put ToFile, , myStyle
    Put ToFile, , myKey
    Put ToFile, , myChance
    Put ToFile, , mySwapTile
    Put ToFile, , myTileSet
    Put ToFile, , myMovementCost
    Put ToFile, , myTerrainType
    Put ToFile, , myNotUsed
End Sub

Public Sub ReadFromFile(FromFile As Integer)
    Dim Cnt As Long, c As Integer
    Dim TriggerX As Trigger
    ' Read Name
    Get FromFile, , myVersion
    Get FromFile, , Cnt
    myName = ""
    For c = 1 To Cnt
        myName = myName & " "
    Next c
    Get FromFile, , myName
    ' Read Index and others
    Get FromFile, , myIndex
    Get FromFile, , myPic
    Get FromFile, , myBlocked
    Get FromFile, , myStyle
    Get FromFile, , myKey
    Get FromFile, , myChance
    Get FromFile, , mySwapTile
    Get FromFile, , myTileSet
    Get FromFile, , myMovementCost
    Get FromFile, , myTerrainType
    Get FromFile, , myNotUsed
End Sub

Public Property Get Key() As Integer
    Key = myKey
End Property

Public Property Let Key(NewValue As Integer)
    myKey = NewValue
End Property

Public Property Get KeyBit(Index As Integer) As Integer
    KeyBit = (myKey And (2 ^ Index)) > 0
End Property

Public Property Let KeyBit(Index As Integer, NewValue As Integer)
    If NewValue Then
        myKey = myKey Or (2 ^ Index)
    Else
        myKey = myKey And (Not (2 ^ Index))
    End If
End Property

Public Property Get KeyBits() As Byte
    KeyBits = (myKey And &HF8)
End Property

Public Property Let KeyBits(NewValue As Byte)
    ' Wipe out the old key
    KeyBits = (myKey And &H7)
    ' Set the new key
    KeyBits = (myKey Or NewValue)
End Property

Public Property Let Style(NewValue As Integer)
    myStyle = CByte(NewValue)
    ' Set other properties if you changed the Sytle
    Select Case myStyle
        Case 0  ' None
        Case 1  ' Left Wall
            Me.Blocked(0) = True
            Me.Blocked(1) = False
            Me.Blocked(2) = False
            Me.Blocked(3) = False
            Me.See(0) = True
            Me.See(1) = False
            Me.See(2) = False
            Me.See(3) = False
        Case 2  ' Top Wall
            Me.Blocked(0) = True
            Me.Blocked(1) = True
            Me.Blocked(2) = False
            Me.Blocked(3) = False
            Me.See(0) = True
            Me.See(1) = True
            Me.See(2) = False
            Me.See(3) = False
        Case 3  ' Floor
            myBlocked = 0
        Case 4  ' Left Arch
            myBlocked = 0
        Case 5  ' Left Door
            Me.Blocked(0) = True
            Me.Blocked(1) = False
            Me.Blocked(2) = False
            Me.Blocked(3) = False
            Me.See(0) = True
            Me.See(1) = False
            Me.See(2) = False
            Me.See(3) = False
            myChance = 100
        Case 10 ' Corner Arch
            Me.Blocked(0) = False
            Me.Blocked(1) = True
            Me.Blocked(2) = False
            Me.Blocked(3) = False
            Me.See(0) = False
            Me.See(1) = True
            Me.See(2) = False
            Me.See(3) = False
        Case 11 ' Block
            Me.Blocked(0) = True
            Me.Blocked(1) = True
            Me.Blocked(2) = True
            Me.Blocked(3) = True
            Me.See(0) = True
            Me.See(1) = True
            Me.See(2) = True
            Me.See(3) = True
    End Select
End Property

Public Property Get Style() As Integer
    Style = CInt(myStyle)
End Property

Public Property Get See(ByVal Index As Integer) As Integer
    Select Case Index
        Case 0  ' Upper Left
            See = (myBlocked And 16) > 0
        Case 1  ' Upper Right
            See = (myBlocked And 32) > 0
        Case 2  ' Lower Left
            See = (myBlocked And 64) > 0
        Case 3  ' Lower Right
            See = (myBlocked And 128) > 0
        Case 4  ' Straight Up (Upper Left and Upper Right)
            See = (myBlocked And 16) > 0 And (myBlocked And 32) > 0
        Case 5  ' Straight Left (Upper Left and Lower Left)
            See = (myBlocked And 16) > 0 And (myBlocked And 64) > 0
        Case 6  ' Straight Right (Upper Right and Lower Right)
            See = (myBlocked And 32) > 0 And (myBlocked And 128) > 0
        Case 7  ' Straight Down (Lower Left and Lower Right)
            See = (myBlocked And 64) > 0 And (myBlocked And 128) > 0
    End Select
End Property

Public Property Let See(ByVal Index As Integer, NewValue As Integer)
    If NewValue Then
        myBlocked = myBlocked Or 2 ^ (Index + 4)
    Else
        myBlocked = myBlocked And (Not 2 ^ (Index + 4))
    End If
End Property

Public Property Get Blocked(ByVal Index As Integer) As Integer
    Select Case Index
        Case 0  ' Upper Left
            Blocked = (myBlocked And &H1) > 0
        Case 1  ' Upper Right
            Blocked = (myBlocked And &H2) > 0
        Case 2  ' Lower Left
            Blocked = (myBlocked And &H4) > 0
        Case 3  ' Lower Right
            Blocked = (myBlocked And &H8) > 0
        Case 4  ' Straight Up (Upper Left and Upper Right)
            Blocked = (myBlocked And &H1) > 0 And (myBlocked And &H2) > 0
        Case 5  ' Straight Left (Upper Left and Lower Left)
            Blocked = (myBlocked And &H1) > 0 And (myBlocked And &H4) > 0
        Case 6  ' Straight Right (Upper Right and Lower Right)
            Blocked = (myBlocked And &H2) > 0 And (myBlocked And &H8) > 0
        Case 7  ' Straight Down (Lower Left and Lower Right)
            Blocked = (myBlocked And &H4) > 0 And (myBlocked And &H8) > 0
    End Select
End Property

Public Property Let Blocked(ByVal Index As Integer, NewValue As Integer)
    If NewValue Then
        myBlocked = myBlocked Or 2 ^ Index
    Else
        myBlocked = myBlocked And (Not 2 ^ Index)
    End If
End Property

Public Property Get BlockedAll() As Byte
    BlockedAll = myBlocked
End Property

Public Property Let BlockedAll(NewValue As Byte)
    myBlocked = NewValue
End Property

Public Property Get Flip() As Integer
    Flip = (myKey And &H1) > 0
End Property

Public Property Let Flip(NewValue As Integer)
    If NewValue Then
        myKey = myKey Or &H1
    Else
        myKey = myKey And (Not &H1)
    End If
End Property

Public Property Get TileSet() As Integer
    TileSet = myTileSet
End Property

Public Property Let TileSet(NewValue As Integer)
    myTileSet = NewValue
End Property

Private Sub Class_Initialize()
    myName = "Untitled"
End Sub
