VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "Topic"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' Version number of Class
Private myVersion As Integer
' Unique number for the Topic
Private myIndex As Integer
' Text of what CreatureWithTurn says
Private mySay As String
' Text of reply
Private myReply As String
' Action to perform after Reply
' 0 - Do nothing
' 1 - Set Package named in ActionTopic as CurrentConvo for NCreatureWithTurn
' 2 - Remove this Topic
' 3 - Add Topic in ActionTopic
' 4 - Remove this Topic and add Topic in ActionTopic
' 5 - Close Convo
Private myAction As Byte
' Reference to Index of Topic in same Convo or to different Convo
Private myActionRef As Integer
' True if shows up as a default, else must be Action que'd.
Private myDefault As Integer
' List of Factoids *required* in Tome for Topic to list in
' Conversation. Must have *all* Factoids for Topic to list.
Private myFactoids As New Collection
' Triggers for this Topic: Post-Reply fires just after Reply is
' displayed; Pre-Reply fires just after click on Topic, before Reply
' is displayed (allows changing this Topic's text based on Statements);
' Pre-ShowTopic fires just before list Topic. Fail indicates do not
' list it (allows checking things with Statements before Topic will
' be available).
Private myTriggers As New Collection

Public Property Get Index() As Integer
    Index = myIndex
End Property

Public Property Let Index(NewIndex As Integer)
    myIndex = NewIndex
End Property

Public Property Get Say() As String
    Say = Trim(mySay)
End Property

Public Property Let Say(NewSay As String)
    mySay = Trim(NewSay)
End Property

Public Property Get Reply() As String
    Reply = Trim(myReply)
End Property

Public Property Let Reply(NewValue As String)
    myReply = Trim(NewValue)
End Property

Public Property Get Action() As Byte
    Action = myAction
End Property

Public Property Let Action(NewValue As Byte)
    myAction = NewValue
End Property

Public Property Get ActionRef() As Integer
    ActionRef = Trim(myActionRef)
End Property

Public Property Let ActionRef(NewValue As Integer)
    myActionRef = Trim(NewValue)
End Property

Public Property Get Default() As Integer
    Default = myDefault
End Property

Public Property Let Default(NewValue As Integer)
    myDefault = NewValue
End Property

Public Property Get Factoids() As Collection
    Set Factoids = myFactoids
End Property

Public Function AddFactoid() As Factoid
    ' Adds an Item and returns a reference to that Item
    Dim c As Integer
    Dim FactoidX As Factoid
    ' Find new available unused index idenifier
    c = 1
    For Each FactoidX In myFactoids
        If FactoidX.Index >= c Then
            c = FactoidX.Index + 1
        End If
    Next FactoidX
    ' Set the index and add the factoid.
    Set FactoidX = New Factoid
    FactoidX.Index = c
    myFactoids.Add Item:=FactoidX, Key:="F" & FactoidX.Index
    Set AddFactoid = FactoidX
End Function

Public Sub RemoveFactoid(DeleteKey As String)
    myFactoids.Remove DeleteKey
End Sub

Public Property Get Triggers() As Collection
    Set Triggers = myTriggers
End Property

Public Function AddTrigger() As Trigger
    ' Adds an Item and returns a reference to that Item
    Dim c As Integer
    Dim TriggerX As Trigger
    ' Find new available unused index idenifier
    c = 1
    For Each TriggerX In myTriggers
        If TriggerX.Index >= c Then
            c = TriggerX.Index + 1
        End If
    Next TriggerX
    ' Set the index and add the door. Return the new door's index.
    Set TriggerX = New Trigger
    TriggerX.Index = c
    TriggerX.Name = "Trigger" & c
    myTriggers.Add Item:=TriggerX, Key:="T" & TriggerX.Index
    Set AddTrigger = TriggerX
End Function

Public Sub RemoveTrigger(DeleteKey As String)
    myTriggers.Remove DeleteKey
End Sub

Public Sub Copy(FromTopic As Topic)
    Dim c As Integer
    Dim FactoidX As Factoid, TriggerX As Trigger
    ' Copy properties
    mySay = FromTopic.Say
    myReply = FromTopic.Reply
    myAction = FromTopic.Action
    myActionRef = FromTopic.ActionRef
    myDefault = FromTopic.Default
    ' Copy Factoids
    Set myFactoids = New Collection
    For c = 1 To myFactoids.Count
        Set FactoidX = Me.AddFactoid
        FactoidX.Copy FromTopic.Factoids(c)
    Next c
    ' Copy Triggers
    Set myTriggers = New Collection
    For c = 1 To FromTopic.Triggers.Count
        Set TriggerX = Me.AddTrigger
        TriggerX.Copy FromTopic.Triggers(c)
    Next c
End Sub

Public Sub ReadFromFile(FromFile As Integer)
    Dim c As Integer, Cnt As Long
    Dim FactoidX As Factoid, TriggerX As Trigger
    ' Read Topic Index, Text, etc
    Get FromFile, , myVersion
    Get FromFile, , myIndex
    mySay = ""
    Get FromFile, , Cnt
    For c = 1 To Cnt
        mySay = mySay & " "
    Next c
    Get FromFile, , mySay
    myReply = ""
    Get FromFile, , Cnt
    For c = 1 To Cnt
        myReply = myReply & " "
    Next c
    Get FromFile, , myReply
    Get FromFile, , myAction
    Get FromFile, , myActionRef
    Get FromFile, , myDefault
    ' Read Factoids for Topic
    Get FromFile, , Cnt
    For c = 1 To Cnt
        Set FactoidX = New Factoid
        FactoidX.ReadFromFile FromFile
        myFactoids.Add Item:=FactoidX, Key:="F" & FactoidX.Index
    Next c
    ' Read Triggers for Topic
    Get FromFile, , Cnt
    For c = 1 To Cnt
        Set TriggerX = New Trigger
        TriggerX.ReadFromFile FromFile
        myTriggers.Add Item:=TriggerX, Key:="T" & TriggerX.Index
    Next c
End Sub

Public Sub SaveToFile(ToFile As Integer)
    Dim c As Integer
    ' Save Topic Name, Index, etc
    Put ToFile, , myVersion
    Put ToFile, , myIndex
    Put ToFile, , Len(mySay)
    Put ToFile, , mySay
    Put ToFile, , Len(myReply)
    Put ToFile, , myReply
    Put ToFile, , myAction
    Put ToFile, , myActionRef
    Put ToFile, , myDefault
    ' Save Factoids for Topic
    Put ToFile, , myFactoids.Count
    For c = 1 To myFactoids.Count
        myFactoids(c).SaveToFile ToFile
    Next c
    ' Save Triggers for Topic
    Put ToFile, , myTriggers.Count
    For c = 1 To myTriggers.Count
        myTriggers(c).SaveToFile ToFile
    Next c
End Sub

Private Sub Class_Initialize()
    mySay = "You say...."
    myDefault = True
    myReply = "He/She replies...."
    myAction = 0    ' Do Nothing
End Sub
